"""Vector Signal Explorer Analog Demod Functions"""
from rssd.VSA.Common import VSA

class VSA(VSA):
    """ Rohde & Schwarz Vector Signal Analyzer Analog Demod Object """
    def __init__(self):
        super(VSA,self).__init__()     #Python2

    #####################################################################
    ### ADemod Get
    #####################################################################
    def Get_Adem_dbw(self):
        """100Hz - MaxBW: Values is rounded up: 1;3;5;"""
        rdStr = self.queryInt('SENS:BWID:DEM?')
        return rdStr

    #####################################################################
    ### VSA Init Settings
    #####################################################################
    def Init_ADemod(self):
        self.Set_Channel("ADEM")

    #####################################################################
    ### ADemod Set
    #####################################################################
    def Set_Adem_Coupling(self,sCouple):
        """AC DC"""
        self.write(f'SENS:ADEM:AF:COUP {sCouple}')

    def Set_Adem_dbw(self,iBW):
        """100Hz - MaxBW: Values is rounded up: 1;3;5;"""
        self.write(f'SENS:BWID:DEM {iBW}')

    def Set_Adem_LPassStat(self,on):
        """LPass Filter: on"""
        if (on == 'ON') or (on == 1):
            self.write('SENSe:FILT:LPASS:STAT ON')
        elif (on == 'OFF') or (on == 0):
            self.write('SENSe:FILT:LPASS:STAT OFF')
        else:                                                   #pragma: no cover
            print('State not supported, please set ON or OFF')

    def Set_Adem_LPassAbsolute(self,sBW):
        """Low Pass Filter Absolute Values: 3kHz; 15kHz; 150kHz"""
        self.write(f'SENSe:FILT:LPASS:FREQ:ABS {sBW}')

    def Set_Adem_LPassRelative(self,sBW):
        """Low Pass Filter Relative Values:5PCT; 10PCT; 25PCT"""
        self.write(f'SENSe:FILT:LPASS:FREQ:REL {sBW}')

    def Set_Adem_LPassManual(self,fBW):
        """0 to 3MHz"""
        self.write(f'SENSe:FILT:LPASS:FREQ:MAN {fBW}')

    def Set_Adem_PM_Unit(self,sUnit):
        """DEG RAD"""
        self.write(f'UNIT:ANGL {sUnit}')

    def Set_Adem_PM_Scale(self,iRet):
        """Degrees per reticle"""
        self.write(f'DISP:TRAC1:Y:PDIV {iRet}')

    def Set_Adem_PM_RefPos(self,iRefPos):
        """PM Reference position: 0-100"""
        self.write(f'DISP:TRAC:Y:SCAL:RPOS {iRefPos}PCT')

    def Set_Adem_PM_RefVal(self,iRefVal):
        """PM Reference Value: PM degrees"""
        self.write(f'DISP:TRAC:Y:SCAL:RVAL {iRefVal}')

#####################################################################
### Run if Main
#####################################################################
if __name__ == "__main__":
    ### this won't be run when imported
    # import sys
    # print(sys.version)
    VSA = VSA()
    VSA.jav_Open("192.168.1.109")
    VSA.Set_DisplayUpdate('ON')
    VSA.Set_Adem_dbw(50e6)
    VSA.Set_InitImm()
    VSA.jav_Close()



###############################################################################
### Purpose: rssd.RCT.NR5G_KM601 test
###              _   ___        __  _____         _
###             | | | \ \      / / |_   _|__  ___| |_
###             | |_| |\ \ /\ / /    | |/ _ \/ __| __|
###             |  _  | \ V  V /     | |  __/\__ \ |_
###             |_| |_|  \_/\_/      |_|\___||___/\__|
###             Please connect instrument prior 2 test
###############################################################################
### User Entry
###############################################################################
host = '192.168.1.160'              #Get local machine name

###############################################################################
### Code Start
###############################################################################
import unittest
from rssd.RCT.NR5G_KM601    import RCT

class TestGeneral(unittest.TestCase):
    def setUp(self):                      #run before each test
        self.CMP = RCT().jav_OpenTest(host)
        self.CMP.Init_5GNR()

    def tearDown(self):                         #Run after each test
        self.assertEqual(self.CMP.jav_Error()[0],'0')
        self.CMP.jav_Close()

###############################################################################
### <Test>
###############################################################################
    def test_CMP_5GNR_ChBW(self):
        self.CMP.Set_5GNR_ChannelBW(200)
        getVal = self.CMP.Get_5GNR_ChannelBW()

    def test_CMP_5GNR_Direction(self):
        self.CMP.Set_5GNR_Direction('UL')
        getVal = self.CMP.Get_5GNR_Direction()
        if self.CMP.connected: self.assertEqual(getVal,'UL')

    def test_CMP_5GNR_FreqRange(self):
        self.CMP.Set_5GNR_Direction('UL')
        self.CMP.Set_5GNR_FreqRange('HIGH')
        getVal = self.CMP.Get_5GNR_FreqRange()
        if self.CMP.connected: self.assertEqual(getVal,'HIGH')

    def test_CMP_5GNR_Ex_EVM_Set(self):
        self.CMP.Set_5GNR_Freq(7e9)
        self.CMP.Set_5GNR_ExpPwr(-50)
        self.CMP.Set_5GNR_UserMargin(0)
        self.CMP.Set_5GNR_MixerOff(0)
        self.CMP.Set_5GNR_ExtAttn(0)
        self.CMP.Set_5GNR_FreqRange(2)       # Function does not exist
        self.CMP.Set_5GNR_Path('P1.RRH.RF1')
        self.CMP.Set_5GNR_BWP_Frame_Periodicity(2)
        self.CMP.Set_5GNR_BWP_Frame_SlotConfig(0,0,8,0)
        self.CMP.Set_5GNR_ChannelBW(100)
        self.CMP.Set_5GNR_CellID(1)
        self.CMP.Set_5GNR_BWP_Ch_DMRS_1stDMRSSym(2)
        # CMP.Set_5GNR_NumBWP()
        self.CMP.Set_5GNR_BWP_ResBlock(66, 0)
        self.CMP.write(f'CONF:NRMM:MEAS:CC1:BWP:PUSC:DMTA BWP0, 1, 2, 1')   #Config; AddPos; MaxLength
        self.CMP.write(f'CONF:NRMM:MEAS:CC1:BWP:PUSC:DMTB BWP0, 1, 2, 1')   #Config; AddPos; MaxLength
        self.CMP.Set_5GNR_TransPrecoding('OFF')
        self.CMP.Set_5GNR_PUSCH(66, 0, 'QPSK')
        self.CMP.write(f'CONF:NRMM:MEAS:CC1:ALL1:PUSC:ADD 1, 2, 3, 0')      #Len; CDM; Pwr; Ant
        self.CMP.write(f'CONF:NRMM:MEAS:CC1:ALL1:PUSC:SGEN CID, 0, 0')      #SeqType; DMRSID; N_SCID
        self.CMP.Set_5GNR_EVM_MeasOnExcept('ON')
        self.CMP.Set_5GNR_EVM_AvgCount(20)
        self.CMP.Set_5GNR_Trigger_Source('Free Run (Fast Sync)')

    def test_CMP_5GNR_Ex_EVM_Get(self):
        self.CMP.Get_AmpSettings()
        self.CMP.Get_5GNR_CC_Offset()
        self.CMP.Get_5GNR_Params(1,1,1,1)           # Get Headers
        self.CMP.Get_5GNR_Params(1,1,1,0)           # Get Data
        self.CMP.Get_5GNR_Params_EVM(1)             # Get Headers
        # self.CMP.Get_5GNR_Params_EVM(0)           # Get Data
        self.CMP.Get_5GNR_EVM()

    def test_CMP_5GNR_Get_UL(self):
        self.CMP.Set_5GNR_Direction('UL')
        # nullVal = self.CMP.Get_5GNR_CC_Freq()
        nullVal = self.CMP.Get_5GNR_Direction()
        nullVal = self.CMP.Get_5GNR_FreqRange()
        nullVal = self.CMP.Get_5GNR_RefA()
        nullVal = self.CMP.Get_5GNR_ChannelBW()
        nullVal = self.CMP.Get_5GNR_TransPrecoding()
        nullVal = self.CMP.Get_5GNR_PhaseCompensate()
        nullVal = self.CMP.Get_5GNR_CellID()
        ### "=User="
        nullVal = self.CMP.Get_5GNR_BWP_SubSpace()
        nullVal = self.CMP.Get_5GNR_BWP_Count()
        self.CMP.Get_5GNR_BWP_ResBlock()
        self.CMP.Get_5GNR_BWP_ResBlockOffset()
        ### "==Ch=="
        self.CMP.Get_5GNR_BWP_Ch_Modulation()
        self.CMP.Get_5GNR_BWP_Ch_ResBlock()
        self.CMP.Get_5GNR_BWP_Ch_ResBlockOffset()
        self.CMP.Get_5GNR_BWP_Ch_SymbNum()
        self.CMP.Get_5GNR_BWP_Ch_SymbOff()
        nullVal = self.CMP.Get_5GNR_BWP_Center()
        ### "=DMRS="
        nullVal = self.CMP.Get_5GNR_BWP_Ch_DMRS_Mapping()
        nullVal = self.CMP.Get_5GNR_BWP_Ch_DMRS_1stDMRSSym()

        nullVal = self.CMP.Get_5GNR_BWP_Ch_DMRS_Config()
        self.CMP.Get_5GNR_BWP_Ch_DMRS_AddPosition()

        nullVal = self.CMP.Get_5GNR_BWP_Ch_DMRS_MSymbLen()
        self.CMP.Get_5GNR_BWP_Ch_DMRS_CDMGroup()
        self.CMP.Get_5GNR_BWP_Ch_DMRS_Antenna()
        self.CMP.Get_5GNR_BWP_Ch_DMRS_RelPwr()
        
        nullVal = self.CMP.Get_5GNR_BWP_Ch_DMRS_SeqGenMeth()
        self.CMP.Get_5GNR_BWP_Ch_DMRS_SeqGenSeed()
        self.CMP.Get_5GNR_BWP_Ch_DMRS_SeqGen_n_SCID()

        ### "=PTRS="
        nullVal = self.CMP.Get_5GNR_BWP_Ch_PTRS_State()
        nullVal = self.CMP.Get_5GNR_BWP_Ch_PTRS_L()
        nullVal = self.CMP.Get_5GNR_BWP_Ch_PTRS_K()
        nullVal = self.CMP.Get_5GNR_BWP_Ch_PTRS_Pow()
        nullVal = self.CMP.Get_5GNR_BWP_Ch_PTRS_RE_Offset()

    def test_CMP_5GNR_Set_DL(self):
        self.CMP.Set_5GNR_Direction('DL')
        self.CMP.Get_5GNR_Direction()
        self.CMP.Get_5GNR_SSB_SubSpace()
    #     self.CMP.Set_5GNR_CC_Num(1)
    #     self.CMP.Set_5GNR_TransPrecoding('OFF')
    #     self.CMP.Set_5GNR_PhaseCompensate('OFF')
    #     self.CMP.Set_5GNR_FreqRange('HIGH')
    #     self.CMP.Set_5GNR_ChannelBW(100)
    #     self.CMP.Set_5GNR_BWP_SubSpace(120)
    #     self.CMP.Set_5GNR_BWP_ResBlock(66)
    #     self.CMP.Set_5GNR_BWP_ResBlockOffset(0)
    #     self.CMP.Set_5GNR_BWP_Ch_ResBlock(66)
    #     self.CMP.Set_5GNR_BWP_Corset_ResBlock(66)
    #     #self.CMP.Set_5GNR_BWP_Ch_ResBlockOffset(NR_RBO)
    #     self.CMP.Set_5GNR_BWP_Ch_Modulation('QPSK')
    #     # self.CMP.Set_5GNR_SSB()

    def test_CMP_5GNR_PhaseComp(self):
        self.CMP.Set_5GNR_PhaseCompensate('ON')
        getVal = self.CMP.Get_5GNR_PhaseCompensate()
        self.CMP.Set_5GNR_PhaseCompensate_Freq(39e9)
        getVal = self.CMP.Get_5GNR_PhaseCompensate()
        getVal = self.CMP.Get_5GNR_PhaseCompensate_Freq()

    def test_CMP_5GNR_TransPrecoding(self):
        self.CMP.Set_5GNR_TransPrecoding('ON')
        getVal = self.CMP.Get_5GNR_TransPrecoding()
        self.CMP.Set_5GNR_TransPrecoding('OFF')
        getVal = self.CMP.Get_5GNR_TransPrecoding()

###############################################################################
### </Test>
###############################################################################
if __name__ == '__main__':
    suite = unittest.TestLoader().loadTestsFromTestCase(TestGeneral)
    unittest.TextTestRunner(verbosity=2).run(suite)



""" Radio Communication Tester(RCT) Functions"""
#pylint: disable=E0611,E0401
from rssd.RCT.Common import RCT

class RCT(RCT):
    """ Rohde & Schwarz Radio Comm Tester Object """
    def __init__(self):
        super(RCT, self).__init__()
        self.Model  = "CMW-GPRF"
        self.sdir   = "UL"
        self.BWP    = 0
        self.User   = 0
        self.alloc  = 1
        self.subF   = 0
        self.cc     = 1         # 1 Start
        self.scs    = 'S120K'

    ############################################################################
    ### RCT Get Functions
    ############################################################################
    def Get_AmpSettings(self):
        """Get ExpectPwr; UserMargin; ExtAttn; MixerOffset settings"""
        expp = self.Get_5GNR_ExpPwr()
        user = self.Get_5GNR_UserMargin()
        exta = self.Get_5GNR_ExtAttn()
        mixr = self.Get_5GNR_MixerOff()
        return f'{expp:7.3f},{user:7.3f},{exta:7.3f},{mixr:2d}'

    def Get_5GNR_EVM(self):
        """ Arry4: EVM_RMS_HIGH
            Arry19:TxPwr
            Arry20:PkPwr
            Arry16:FrqErr"""
        # self.write('ABOR:NRMM:MEAS:MEV')
        self.query('INIT:NRMM:MEAS:MEV;*OPC?')                                  #RUN state
        rdStr = self.queryFloatArry('FETC:NRMM:MEAS:MEV:MOD:AVER?')
        try:
            rdStr = [rdStr[2], rdStr[18], rdStr[19], rdStr[15]]
        except:
            rdStr = [-9999,-9999,-9999,-9999]
        return rdStr

    def Get_5GNR_ExpPwr(self):
        """ExpPwr = Range + ExtAttn - UserMargin
           units: dBm """
        rdStr = self.queryFloat(f'CONF:NRMM:MEAS:RFS:ENP?')
        return rdStr

    def Get_5GNR_ExtAttn(self):
        """ExpPwr = Range + ExtAttn - UserMargin
           units: dB """
        rdStr = self.queryFloat(f'CONF:NRMM:MEAS:RFS:EATT?')
        return rdStr

    def Get_5GNR_MixerOff(self):
        """ExpPwr = Range + ExtAttn - UserMargin
           range: -10 to 10"""
        rdStr = self.queryInt(f'CONF:NRMM:MEAS:RFS:MLOF?')
        return rdStr

    def Get_5GNR_UserMargin(self):
        """ExpPwr = Range + ExtAttn - UserMargin
           units: dB """
        rdStr = self.queryFloat(f'CONF:NRMM:MEAS:RFS:UMAR?')
        return rdStr

    ###########################################################################
    ### 5GNR Get Methods
    ###########################################################################
    def Get_5GNR_BWP_Center(self):
        rdStr = "<notRead>"
        return rdStr

    def Get_5GNR_BWP_Ch_DMRS_1stDMRSSym(self):
        """2 3 """
        rdStr = self.query(f'CONF:NRMM:MEAS:CC{self.cc}:TAP?')
        return rdStr

    def Get_5GNR_BWP_Ch_DMRS_AddPosition(self):
        """ 0 to 3 """
        rdStr = self.queryFloatArry(f'CONF:NRMM:MEAS:CC{self.cc}:BWP:PUSC:DMTA?  BWP{self.BWP}')
        return rdStr[1]

    def Get_5GNR_BWP_Ch_DMRS_Antenna(self):
        """ 1 to 0 """
        rdStr = self.queryFloatArry(f'CONF:NRMM:MEAS:CC{self.cc}:ALL{self.alloc}:PUSC:ADD?')
        return rdStr[3]

    def Get_5GNR_BWP_Ch_DMRS_CDMGroup(self):
        """ 1 to 3 """
        rdStr = self.queryFloatArry(f'CONF:NRMM:MEAS:CC{self.cc}:ALL{self.alloc}:PUSC:ADD?')
        return rdStr[1]

    def Get_5GNR_BWP_Ch_DMRS_Config(self):
        """ 1 to 2 """
        rdStr = self.queryFloatArry(f'CONF:NRMM:MEAS:CC{self.cc}:BWP:PUSC:DMTA? BWP{self.BWP}')
        return rdStr[0]

    def Get_5GNR_BWP_Ch_DMRS_Mapping(self):
        rdStr = self.queryFloatArry(f'CONF:NRMM:MEAS:CC{self.cc}:ALL{self.alloc}:PUSC?')
        return rdStr[0]

    def Get_5GNR_BWP_Ch_DMRS_MSymbLen(self):
        rdStr = self.queryFloatArry(f'CONF:NRMM:MEAS:CC{self.cc}:ALL{self.alloc}:PUSC:ADD?')
        return rdStr[0]

    def Get_5GNR_BWP_Ch_DMRS_RelPwr(self):
        """ -10 to 10 """
        rdStr = self.queryFloatArry(f'CONF:NRMM:MEAS:CC{self.cc}:ALL{self.alloc}:PUSC:ADD?')
        return rdStr[2]

    def Get_5GNR_BWP_Ch_DMRS_SeqGenMeth(self):
        """CellID DMRS"""
        rdStr = self.queryFloatArry(f'CONF:NRMM:MEAS:CC{self.cc}:ALL{self.alloc}:PUSC:SGEN?')
        return rdStr[0]

    def Get_5GNR_BWP_Ch_DMRS_SeqGenSeed(self):
        """ 0 to 655535 """
        rdStr = self.queryFloatArry(f'CONF:NRMM:MEAS:CC{self.cc}:ALL{self.alloc}:PUSC:SGEN?')
        return rdStr[1]

    def Get_5GNR_BWP_Ch_DMRS_SeqGen_n_SCID(self):
        """ 0 to 1 """
        rdStr = self.queryFloatArry(f'CONF:NRMM:MEAS:CC{self.cc}:ALL{self.alloc}:PUSC:SGEN?')
        return rdStr[2]

    def Get_5GNR_BWP_Ch_Modulation(self):
        rdStr = self.queryFloatArry(f'CONF:NRMM:MEAS:CC{self.cc}:ALL{self.alloc}:PUSC?')
        return rdStr[5]

    def Get_5GNR_BWP_Ch_Mapping(self):
        """PUSCH Mapping"""
        rdStr = self.queryFloatArry(f'CONF:NRMM:MEAS:CC{self.cc}:ALL{self.alloc}:PUSC?')
        return rdStr[0]

    def Get_5GNR_BWP_Ch_PTRS_K(self):
        """ Freq Density in RB """
        rdStr = "<TBS>"
        return rdStr

    def Get_5GNR_BWP_Ch_PTRS_L(self):
        """ Time Density in OFDM Sym Freq """
        rdStr = "<TBS>"
        return rdStr

    def Get_5GNR_BWP_Ch_PTRS_Pow(self):
        rdStr = "<TBS>"
        return rdStr

    def Get_5GNR_BWP_Ch_PTRS_RE_Offset(self):
        """ PTRS freq (RE) offset """
        rdStr = "<TBS>"
        return rdStr

    def Get_5GNR_BWP_Ch_PTRS_State(self):
        rdStr = "<TBS>"
        return rdStr

    def Get_5GNR_BWP_Ch_ResBlock(self):
        ### RB = (CHBw * 0.95) / (SubSp * 12)
        rdStr = self.queryFloatArry(f'CONF:NRMM:MEAS:CC{self.cc}:ALL{self.alloc}:PUSC?')
        return rdStr[3]

    def Get_5GNR_BWP_Ch_ResBlockOffset(self):
        rdStr = self.queryFloatArry(f'CONF:NRMM:MEAS:CC{self.cc}:ALL{self.alloc}:PUSC?')
        return rdStr[4]

    def Get_5GNR_BWP_Ch_SymbNum(self):
        rdStr = self.queryFloatArry(f'CONF:NRMM:MEAS:CC{self.cc}:ALL{self.alloc}:PUSC?')
        return rdStr[1]

    def Get_5GNR_BWP_Ch_SymbOff(self):
        rdStr = self.queryFloatArry(f'CONF:NRMM:MEAS:CC{self.cc}:ALL{self.alloc}:PUSC?')
        return rdStr[2]

    def Get_5GNR_BWP_Count(self):
        rdStr = 1
        return rdStr

    def Get_5GNR_BWP_ResBlock(self):
        ### RB = (CHBw * 0.95) / (SubSp * 12)
        rdStr = self.queryFloatArry(f'CONF:NRMM:MEAS:CC{self.cc}:BWP? BWP{self.BWP}')
        return rdStr[2]

    def Get_5GNR_BWP_ResBlockOffset(self):
        rdStr = self.queryFloatArry(f'CONF:NRMM:MEAS:CC{self.cc}:BWP? BWP{self.BWP}')
        return rdStr[3]

    def Get_5GNR_BWP_SubSpace(self):
        rdStr = self.queryFloatArry(f'CONF:NRMM:MEAS:CC{self.cc}:BWP? BWP{self.BWP}')
        return rdStr[0]

    def Get_5GNR_CC_Offset(self):
        rdStr = "<notRead>"
        return rdStr

    def Get_5GNR_CellID(self):
        rdStr = self.query(f'CONF:NRMM:MEAS:CC{self.cc}:PLC?')
        return rdStr

    def Get_5GNR_CrestFactor(self):
        self.query('INIT:NRMM:MEAS:MEV;*OPC?')                                        #RUN state.
        rdStr = self.queryFloatArry(f'FETC:NRMM:MEAS:MEV:MOD:AVER?')
        output = rdStr[19] - rdStr[18]
        return output

    def Get_5GNR_SSB_SubSpace(self):
        if self.sdir == 'DL':
            # rdStr = self.query(f':SOUR1:BB:NR5G:NODE:CELL{self.cc}:SSPB0:SCSP?')
            pass
        else:
            rdStr = '<UL n/a>'
        return rdStr

    def Get_5GNR_ChannelBW(self):
        ### 5;10;15;20;25;30;40;50;60;70;80;90;100;200;400
        rdStr = self.query(f'CONF:NRMM:MEAS:CC{self.cc}:CBAN?')
        return rdStr

    def Get_5GNR_Direction(self):
        rdStr = "UP"
        if rdStr == 'DOWN':
            self.sdir = "DL"
            self.alloc = 1                                                      #Alloc 0:CORSET
        elif rdStr == 'UP':
            self.sdir = "UL"
            self.alloc = 1                                                      #Alloc 0:PUSCH
        else:
            print('Get_5GNR_Direvmection Error')
        return self.sdir

    def Get_5GNR_FreqRange(self):
        return "HIGH"

    def Get_5GNR_RefA(self):
        rdStr = '<notRead>'
        return rdStr

    def Get_5GNR_Params(self,amp,swp,sys,header=0):                 #pylint: disable=R0913
        """Amp,Sweep,System,Trace"""
        outStr = ""
        outStr += self.Get_5GNR_Params_Amp(header)+","   if (amp==1) else ""
        outStr += self.Get_5GNR_Params_Sweep(header)+"," if (swp==1) else ""
        outStr += self.Get_5GNR_Params_System(header)+","if (sys==1) else ""
        return outStr

    def Get_5GNR_Params_Amp(self,header=0):
        """Retrieve Parameters for test logs"""
        if header != 1:
            attn    = self.Get_5GNR_UserMargin()
            prea    = self.Get_5GNR_ExtAttn()
            refl    = self.Get_5GNR_ExpPwr()
            outStr  = f'{attn:7.3f},{prea},{refl:7.3f}'
        else:
            outStr = 'UserMargin,ExtAttn,ExpPwr'
        return outStr

    def Get_5GNR_Params_Sweep(self,header=0):
        # SwpTime,SwpPts,SwpType,SwpOpt,
        if header != 1:
            Time    = 1.212904298
            Points  = '<N/A>'
            Type    = '<N/A>'
            Opt     = '<N/A>'
            outStr  = f'{Time:5.3f},{Points},{Type},{Opt}'
        else:
            outStr  = 'SwpTimeM,SwpPts,SwpType,SwpOpt'
        return outStr

    def Get_5GNR_Params_System(self,header=0):
        if header != 1:
            error  = self.jav_Error()
            ext    = self.jav_Error()
            outStr = f'{error[0]:>4},{error[1]:10.10},{ext:10.10}'
        else:
            outStr  = 'ErrNo,ErrMsg,ExtError'
        return outStr

    def Get_5GNR_Params_EVM(self,header=0):
        if header != 1:
            self.query('INIT:NRMM:MEAS:MEV;*OPC?')                              #RUN state
            rdStr = self.queryFloatArry('FETC:NRMM:MEAS:MEV:MOD:AVER?')
            Power   = rdStr[18]     #19: RMS Ch Power
            Crest   = rdStr[19] - Power
            EVMAll  = rdStr[4]      #5: EVM PEAK Low
            EVMPhyC = rdStr[2]      #3: EVM RMS Low
            EVMPhyS = rdStr[21]     #22: EVM DMRS Low
            outStr  = f"{Crest:6.3f},{Power:6.3f},{EVMAll:.2f},{EVMPhyC:.2f},{EVMPhyS:.2f}"
        else:
            outStr  = 'CMP_Cres,CMPPwr,5GEVM_Peak,EVM_RMS,EVM_DMRS'
        return outStr

    def Get_5GNR_PhaseCompensate(self):
        rdStr = self.query(f'CONF:NRMM:MEAS:MEV:PCOM?').split(',')[0]
        if (rdStr == 'CAF') or (rdStr == 'UDEF'):
            outStr = 'ON'
        else:
            outStr = '<notRead>'
        return outStr

    def Get_5GNR_PhaseCompensate_Freq(self):
        rdStr = self.queryIntArry(f'CONF:NRMM:MEAS:MEV:PCOM?')[1]
        return rdStr

    def Get_5GNR_TransPrecoding(self):
        # SC-FDMA or DFT-S-OFDM
        rdStr = self.query(f'CONF:NRMM:MEAS:CC{self.cc}:BWP:PUSC:DFTP? BWP{self.BWP}')
        return rdStr

    ###########################################################################
    ### BSE Init Functions
    ###########################################################################
    def Init_5GNR(self):
        self.write('SYST:GEN:ALL:OFF')
        self.write('SYST:MEAS:ALL:OFF')
        self.write('ABOR:NRMM:MEAS:MEV')                                        #OFF state
        # self.write(f'STOP:NRMM:MEAS:MEV STOP')                                #RDY state
        self.query('INIT:NRMM:MEAS:MEV;*OPC?')                                  #RUN state

    ###########################################################################
    ### BSE Set Functions
    ###########################################################################

    def Set_5GNR_ExpPwr(self, pwr):
        """ExpPwr = Range + ExtAttn - UserMargin
           units: dBm """
        self.write(f'CONF:NRMM:MEAS:RFS:ENP {pwr} DBM')

    def Set_5GNR_ExtAttn(self, pwr):
        """ExpPwr = Range + ExtAttn - UserMargin
           units: dB """
        self.write(f'CONF:NRMM:MEAS:RFS:EATT {pwr} DB')

    def Set_5GNR_Freq(self,freq):
        """freq in Hz"""
        self.write(f'CONF:NRMM:MEAS:RFS:FREQ {int(freq)}')

    def Set_5GNR_MixerOff(self, pwr):
        """ExpPwr = Range + ExtAttn - UserMargin
           range: -10 to 10
           units: dB """
        self.write(f'CONF:NRMM:MEAS:RFS:MLOF {pwr}')

    def Set_5GNR_PhaseComp(self,state,freq):
        """ State: OFF | CAF | UDEF
            Freq : Hz"""
        self.write(f'CONF:NRMM:MEAS:MEV:PCOM {state},{int(freq)}')

    def Set_5GNR_UserMargin(self, pwr):
        """ExpPwr = Range + ExtAttn - UserMargin
           units: dB """
        self.write(f'CONF:NRMM:MEAS:RFS:UMAR {pwr} DB')

    def Set_5GNR_BWP_Ch_DMRS_1stDMRSSym(self,TAP):
        """2 3"""
        self.write(f'CONF:NRMM:MEAS:CC{self.cc}:TAP {TAP}')

    # def Set_5GNR_BWP_Ch_Modulation(self,sMod):
    #     self.write(f':SOUR1:BB:NR5G:SCH:CELL{self.cc}:SUBF{self.subF}:USER0:BWP0:ALL{self.alloc}:MOD {sMod}')

    # def Set_5GNR_BWP_Ch_ResBlock(self,iRB):
    #     ### 5GNR-->Scheduling-->PUSCH-->No. RBs
    #     ### RB = (CHBw * 0.95) / (SubSp * 12)
    #     #self.write(f':SOUR1:BB:NR5G:SCH:CELL{self.cc}:SUBF{self.subF}:USER0:BWP0:ALL{self.alloc}:RBN %d'%iRB)
    #     self.write(f':SOUR1:BB:NR5G:SCH:CELL{self.cc}:SUBF{self.subF}:USER0:BWP0:ALL{self.alloc}:RBN {iRB}')

    # def Set_5GNR_BWP_Ch_ResBlockOffset(self,iRBO):
    #     ### 5GNR-->Scheduling-->PUSCH-->No. RBs
    #     #self.write(f':SOUR1:BB:NR5G:SCH:CELL{self.cc}:SUBF{self.subF}:USER0:BWP0:ALL{self.alloc}:RBOF %d'%%(self.alloc,iRBO))
    #     self.write(f':SOUR1:BB:NR5G:SCH:CELL{self.cc}:SUBF{self.subF}:USER0:BWP0:ALL{self.alloc}:RBOF 0')

    # def Set_5GNR_BWP_Corset_ResBlock(self, iRB):
    #     if self.sdir == 'DL':
    #         self.write(f':SOUR1:BB:NR5G:SCH:CELL{self.cc}:SUBF{self.subF}:USER0:BWP0:ALL0:RBN {iRB}')

    # def Set_5GNR_BWP_Corset_ResBlockOffset(self,iRBO):
    #     if self.sdir == 'DL':
    #     ### 5GNR-->Scheduling-->PUSCH-->No. RBs
    #         self.write(f':SOUR1:BB:NR5G:SCH:CELL{self.cc}:SUBF{self.subF}:USER0:BWP0:ALL0:RBOF {iRBO}')

    def Set_5GNR_BWP_Frame_SlotConfig(self,DLSlot,DLSym,ULSlot,ULSym):
        """DLSlot,DLSym,ULSlot,ULSym """
        self.write(f'CONF:NRMM:MEAS:ULDL:PATT {self.scs}, {DLSlot},{DLSym},{ULSlot},{ULSym}')    #DL Slot; DL Sym; UL SLot; UL Sym

    def Set_5GNR_BWP_Frame_Periodicity(self,msec):
        """ Period,msec: 05 | 0625 | 1 | 125 | 2 | 25 | 5 | 10 """
        self.write(f'CONF:NRMM:MEAS:ULDL:PER MS{msec}')

    def Set_5GNR_BWP_ResBlock(self, iRB, iRBO):
        ### RB = (CHBw * 0.95) / (SubSp * 12)
        self.write(f'CONF:NRMM:MEAS:CCAL:TXBW:SCSP {self.scs}')
        self.write(f'CONF:NRMM:MEAS:CC{self.cc}:BWP BWP{self.BWP}, {self.scs}, NORM, {iRB}, {iRBO}')    #SCS; NORM; RB; RBO

    # def Set_5GNR_BWP_ResBlockOffset(self,iRBO):
    #     self.write(f':SOUR1:BB:NR5G:UBWP:USER0:CELL{self.cc}:{self.sdir}:BWP0:RBOF {iRBO}')

    def Set_5GNR_BWP_SubSpace(self,iSubSp):
        """60| 120"""
        self.scs = f'S{iSubSp}K'
        self.write(f'CONF:NRMM:MEAS:CCAL:TXBW:SCSP {self.scs}')

    # def Set_5GNR_CC_Num(self,iCC):
    #     """ iCC, 1 start """
    #     self.cc = iCC - 1
    #     self.write(f'SOUR1:BB:NR5G:NODE:NCAR {iCC}')

    # def Set_5GNR_CC_Offset(self,Freq):
    #     """ freq, Hz """
    #     self.write(f':SOUR1:BB:NR5G:NODE:CARM:DFR:ROW{self.cc} {Freq}')

    def Set_5GNR_CellID(self,cell):
        """ 0 to 1007"""
        self.write(f'CONF:NRMM:MEAS:CC{self.cc}:PLC {cell}')

    def Set_5GNR_ChannelBW(self,iBW):
        """ 050 100 200 400"""
        self.write(f'CONF:NRMM:MEAS:CC1:CBAN B{iBW}')

    def Set_5GNR_Direction(self,sDirection):
        """ UP| DOWN """
        if (sDirection == "UL") or (sDirection == "UP"):
            # self.write(f':SOUR1:BB:NR5G:LINK UP')
            self.sdir = "UL"
            self.alloc = 1         #Alloc 0:PUSCH
        elif (sDirection == "DL") or (sDirection == "DOWN"):
            # self.write(f':SOUR1:BB:NR5G:LINK DOWN')
            self.sdir = "DL"
            self.alloc = 1         #Alloc 0:Coreset 1:PDSCH
        else:
            if self.debug: print("Set_5GNR_Direction must be UP or DOWN")

    def Set_5GNR_EVM_AvgCount(self, avg):
        """1 to 1000 slots"""
        self.write(f':CONF:NRMM:MEAS:MEV:SCO:MOD {avg}')

    def Set_5GNR_EVM_MeasOnExcept(self, state):
        """ 'ON' | 'OFF' """
        if (state == "ON") or (state == 1):
            self.write(f'CONF:NRMM:MEAS:MEV:MOEX ON')
        else:
            self.write(f'CONF:NRMM:MEAS:MEV:MOEX OFF')

    def Set_5GNR_FreqRange(self, state):
        pass

    def Set_5GNR_Path(self,path):
        """string P1.RRH.RF1 P1.RRH.RF2 """
        self.write(f':ROUT:NRMM:MEAS:SPAT "{path}"')

    def Set_5GNR_PhaseCompensate(self,state):
        """ 'ON' | 'OFF' """
        if (state == "ON") or (state == 1):
            self.write(f'CONF:NRMM:MEAS:MEV:PCOM CAF, 28e9')
        else:
            self.write(f'CONF:NRMM:MEAS:MEV:PCOM OFF, 28e9')

    def Set_5GNR_PhaseCompensate_Freq(self,Freq):
        self.write(f'CONF:NRMM:MEAS:MEV:PCOM UDEF, {Freq}')

    def Set_5GNR_PUSCH(self,RB,RBO,mod):
        if (mod == 'QPSK'):
            mod = 'QPSK'
        elif (mod == 'Q16') or (mod == 'QAM16'):
            mod = 'Q16'
        elif (mod == 'Q64') or (mod == 'QAM64'):
            mod = 'Q64'
        else:
            print(f'Set_5GNR_PUSCH {mod} not supported')
        self.write(f'CONF:NRMM:MEAS:CC{self.cc}:ALL{self.alloc}:PUSC A, 14, 0, {RB}, {RBO}, {mod}')   #Map; Sym; SymStrt; RB; RBO; Mod

    # def Set_5GNR_SSB(self):
    #     """Num DL SS/PBCH Patterns"""
    #     if self.sdir == 'DL':
    #         self.write(f':SOUR1:BB:NR5G:NODE:CELL{self.cc}:NSSP 1')

    def Set_5GNR_TransPrecoding(self, sState):
        """ SC-FDMA or DFT-S-OFDM  """
        if sState in  (1, '1','ON'):
            self.write(f'CONF:NRMM:MEAS:CC{self.cc}:BWP:PUSC:DFTP BWP{self.BWP}, ON')
        elif sState in  (0, '0','OFF'):
            self.write(f'CONF:NRMM:MEAS:CC{self.cc}:BWP:PUSC:DFTP BWP{self.BWP}, OFF')
        else:
            print('Error Set_5GNR_TransPrecoding')

    def Set_5GNR_Trigger_Source(self, source):
        """ string 'Free Run (Fast Sync)' 'Free Run (No Sync)' 'IF Power' """
        self.write(f':TRIG:NRMM:MEAS:MEV:SOUR "{source}"')

    def Set_5GNR_Stop(self):
        self.write(f'STOP:NRMM:MEAS:MEV STOP')                                  #RDY state

###############################################################################
### Run if Main
###############################################################################
if __name__ == "__main__":
    ### this won't be run when imported
    CMP = RCT()
    CMP.jav_Open("192.168.1.160")
    CMP.Set_5GNR_Path('P1.RRH.RF1')
    CMP.Set_5GNR_Freq(28e9)
    CMP.Set_5GNR_ExpPwr(-9)
    CMP.Set_5GNR_UserMargin(13)
    CMP.Set_5GNR_PhaseCompensate_Freq(28e9)
    CMP.Set_5GNR_BWP_Frame_Periodicity('2')
    CMP.Set_5GNR_BWP_Frame_SlotConfig(0,0,8,0)
    CMP.Set_5GNR_ChannelBW(100)
    CMP.Set_5GNR_CellID(1)
    CMP.Set_5GNR_BWP_Ch_DMRS_1stDMRSSym(2)
    # CMP.Set_5GNR_NumBWP()
    CMP.write(f'CONF:NRMM:MEAS:CC{CMP.cc}:BWP BWP0, {CMP.scs}, NORM, 66, 0')    #SCS; NORM; RB; RBO
    CMP.write(f'CONF:NRMM:MEAS:CC{CMP.cc}:BWP:PUSC:DMTA BWP0, 1, 2, 1')         #Config; AddPos; MaxLength
    CMP.write(f'CONF:NRMM:MEAS:CC{CMP.cc}:BWP:PUSC:DMTB BWP0, 1, 2, 1')         #Config; AddPos; MaxLength
    CMP.Set_5GNR_TransPrecoding('OFF')
    CMP.Set_5GNR_PUSCH(22,22,'Q64')
    CMP.write(f'CONF:NRMM:MEAS:CC{CMP.cc}:ALL{CMP.alloc}:PUSC:ADD 1, 2, 3, 0')  #Len; CDM; Pwr; Ant
    CMP.write(f'CONF:NRMM:MEAS:CC{CMP.cc}:ALL{CMP.alloc}:PUSC:SGEN CID, 0, 0')  #SeqType; DMRSID; N_SCID
    CMP.Set_5GNR_EVM_AvgCount(50)
    CMP.Set_5GNR_Trigger_Source('Free Run (Fast Sync)')
    print(CMP.Get_5GNR_EVM())
    CMP.Set_5GNR_Stop()
    CMP.jav_ClrErr()
    CMP.jav_Close()



""" Rohde&Schwarz Python FSW; SMW; NRP; NRQ; OSP; VSA; VSG; VST; driver example code """

__author__ = "Martin C Lim <martin.lim@rsa.rohde-schwarz.com>"
# __all__ = ['FileIO', 'IQ', 'NRQ', 'OSP', 'PMr', 'RCT', 'VNA', 'VSA', 'VSE', 'VSG', 'jaVisa']

from .FileIO      import FileIO
from .iqdata      import IQ
# from .NRQ.Common  import NRQ
# from .OSP.Common  import OSP
# from .NRP.Common  import PMr
# from .RCT.Common  import RCT
# from .VSA.Common  import VSA
# from .VSG.Common  import VSG
# from .VNA.Common  import VNA
# from .VSE.Common  import VSE



import pandas as pd
import matplotlib.pyplot as plt

f = __file__.split('_PLOT')[0] + '.txt'
f = 'VSA_BWMarker.py-210426.csv'
print(f)
df = pd.read_csv(f, header=0, skiprows=1, sep=',')
print(f'Cols: {list(df.columns)}')                  # Col names

# ## Define Pivot Table
Yval = ['MkrBndPwr']                                # Y Values
Xval = ['RBW']                                      # X Values
Cols = ['SwpTime']                                  # Separate Lines
aggg = 'mean'                                       # mean | sum
table = pd.pivot_table(df, values=Yval, index=Xval, columns=Cols, aggfunc=aggg)
print(f'Traces:{table.shape[1]} DataPts:{table.shape[0]}')

# ##############################################################################
# ## Plot Data
# ##############################################################################
table.plot(legend=True)
plt.legend(loc='upper center', bbox_to_anchor=(0.5, -0.1), ncol=3)
plt.tight_layout(pad=2)
plt.grid()
plt.title(f'{Yval} {aggg}')
# plt.axis([-50, 10, -60, -20])                     # X, Y
plt.ylim((-88, -82))
plt.xlabel(Xval)
plt.ylabel(Yval)

plt.savefig(f'{f}.png')
plt.show(block=False)
plt.pause(3)
plt.close()



###############################################################################
### Rohde & Schwarz Automation for demonstration use.
### Capture EVM based on SMW *.nr5G & FSW *.allocation files
#pylint: disable=E0611,E0401
###############################################################################
SMW_IP      = '192.168.1.114'
FSW_IP      = '192.168.1.109'
UserDir     = '2020.07.30-Autolevel'
freqArry    = [28e9]
pwrArry     = range(-50,10,2)                               #Power Array
comment     = '-Autolevel'

###############################################################################
### Overhead
###############################################################################
from rssd.VSG.NR5G_K144     import VSG
from rssd.VSA.NR5G_K144     import VSA
from rssd.FileIO            import FileIO
from rssd.RSI.time          import timer

OFile = FileIO().makeFile(__file__)
TMR = timer()
SMW = VSG().jav_Open(SMW_IP,OFile)                          #Create SMW Object
SMW.debug = 0
FSW = VSA().jav_Open(FSW_IP,OFile)                          #Create FSW Object
FSW.debug = 0

class dataClass():
    def __init__(self):
#         self.Direction      = 'UL'
#         self.CellID         = 1
#         # self.FreqRng      = 'HIGH'
#         self.ChBW           = 100
#         self.TF             = 'ON'
#         self.SubSp          = 120
#         self.RB             = 60
#         self.RBO            = 0
#         self.Ch_RB          = 60
#         self.Ch_RBO         = 0
#         self.Mod            = 'QPSK'
        self.Rx             = ''
#         self.pwr            = -100

def NR5G_Rx_Init():
    """Start 5GNR Measurement Channel"""
    FSW.Init_5GNR()
    FSW.Set_5GNR_FrameCount('OFF')

def NR5G_Rx_Config(sSetting):
    sSetting = sSetting.split('.nr5g')[0]
    FSW.Set_5GNR_AllocFile(f'C:\\R_S\\instr\\user\\Demo\\{UserDir}\\{sSetting}.allocation')
    FSW.Set_5GNR_FrameCount('OFF')
    FSW.Set_Trig1_Source('EXT')
    FSW.Set_SweepTime(3e-3)
    FSW.Set_5GNR_SubFrameCount(16)
    NR5G.Rx = FSW.Model + comment

def NR5G_Rx_SetFreq(freq):
    FSW.Set_Freq(freq)
    FSW.Set_5GNR_PhaseCompensate_Freq(freq)

def NR5G_Rx_SetLevel():
    FSW.Set_SweepCont(1)
    FSW.Set_Autolevel()
    # FSW.Set_5GNR_AutoEVM()

def NR5G_Rx_Get_EVM():
    FSW.Set_SweepCont(0)
    FSW.Set_InitImm()
    EVM = FSW.Get_5GNR_Params_EVM()
    return EVM

###############################################################################
### Code Start
###############################################################################
LoopParam   = 'State,Model,SMW_Fre,SMW_Pwr'
# WaveParam   = 'ChBW,SubSp,RB,Mod,TF'
AttnParam   = FSW.Get_Params_Amp(1)
EVMParam    = FSW.Get_5GNR_Params_EVM(1)
TimeParam   = 'AlTime,MeasTime,TotalTime,HoursLeft'
Header      = f'{LoopParam},{AttnParam},{EVMParam},{TimeParam}'
OFile.write(Header)

### Instr Init
NR5G        = dataClass()
SMW.Set_OS_Dir(UserDir)
SMW.Set_5GNR_BBState(1)
SMW.Set_RFPwr(-50)
SMW.Set_RFState(1)
saveArry = SMW.Get_OS_FileList('savrcltxt')
saveArry = SMW.Get_OS_FileList('nr5g')
NR5G_Rx_Init()

TMR.numTest = len(saveArry) * len(freqArry) * len(pwrArry)
TMR.suite_start()
for saveState in saveArry:
    SMW.Set_5GNR_Setting_Load(f'{UserDir}/{saveState}')
    NR5G_Rx_Config(saveState)
    for freq in freqArry:
        SMW.Set_Freq(freq)
        SMW.Set_5GNR_PhaseCompensate_Freq(freq)
        NR5G_Rx_SetFreq(freq)
        for pwr in pwrArry:
            TMR.start()
            SMW.Set_RFPwr(pwr)
            # SMW.Set_OptimizeAll()
            # NR5G.pwr = pwr
            NR5G_Rx_SetLevel()
            TMR.tick()
            EVM = NR5G_Rx_Get_EVM()
            TMR.tick()

            ### Log Data
            LoopParam   = f'{saveState},{NR5G.Rx},{freq},{pwr:3d}'
            AttnParam   = FSW.Get_Params_Amp()
            TimeParam   = TMR.Get_Params_Time()
            OutStr      = f'{LoopParam},{AttnParam},{EVM},{TimeParam}'
            OFile.write(OutStr)
SMW.Set_RFPwr(-100)



"""Vector Signal Demodulation Functions"""
from rssd.VSA.Common import VSA                 #pylint: disable=E0611,E0401

class VSA(VSA):                                 #pylint: disable=E0102
    """ Rohde & Schwarz Vector Signal Analyzer Object """
    def __init__(self):
        super(VSA, self).__init__()             #Python 2/3
        self.mod = ''

    ###########################################################################
    ### VSA Get Functions
    ###########################################################################
    def Get_VSA_EVM(self):
        rdStr = self.queryFloat(':CALC2:MARK:FUNC:DDEM:STAT:EVM?')
        return rdStr

    def Get_VSA_EqualizerState(self):
        """ ON; OFF"""
        rdStr = self.query(f':SENS:DDEM:EQU:STAT?')
        return rdStr

    def Get_VSA_CarrierFreqError(self):
        rdStr = self.queryFloat(':CALC2:MARK:FUNC:DDEM:STAT:CFER?')
        return rdStr

    def Get_VSA_Capture_Length(self):
        '''Get Capture Symbols'''
        rdStr = self.queryInt(f':SENS:DDEM:RLEN:SYMB?')
        return rdStr

    def Get_VSA_IQImbalance(self):
        rdStr = self.queryFloat(':CALC2:MARK:FUNC:DDEM:STAT:IQIM?')
        return rdStr

    def Get_VSA_GainImbalance(self):
        rdStr = self.queryFloat(':CALC2:MARK:FUNC:DDEM:STAT:GIMB?')
        return rdStr

    def Get_VSA_IQOffset(self):
        rdStr = self.queryFloat(':CALC2:MARK:FUNC:DDEM:STAT:OOFF?')
        return rdStr

    def Get_VSA_IQSkew(self):
        rdStr = self.queryFloat(':CALC2:MARK:FUNC:DDEM:STAT:IQSK?')
        return rdStr

    def Get_VSA_MagnitudeError(self):
        rdStr = self.queryFloat(':CALC2:MARK:FUNC:DDEM:STAT:MERR?')
        return rdStr

    def Get_VSA_Meas_Params(self):
        EVM     = self.Get_VSA_EVM()
        PhaseEr = self.Get_VSA_PhaseError()
        MagEr   = self.Get_VSA_MagnitudeError()
        FreqEr  = self.Get_VSA_CarrierFreqError()
        IQOff   = self.Get_VSA_IQOffset()
        return f'{EVM:.4f},{PhaseEr:.4f},{MagEr:.4f},{FreqEr:.4f},{IQOff:.4f}'

    def Get_VSA_MER(self):
        rdStr = self.queryFloat(':CALC2:MARK:FUNC:DDEM:STAT:SNR?')
        return rdStr

    def Get_VSA_PhaseError(self):
        rdStr = self.queryFloat(':CALC2:MARK:FUNC:DDEM:STAT:PERR?')
        return rdStr

    def Get_VSA_ResultSumamry(self):
        rdStr = self.query(':TRACE2:DATA? TRACE1')
        return rdStr

    def Get_VSA_Rho(self):
        rdStr = self.queryFloat(':CALC2:MARK:FUNC:DDEM:STAT:RHO?')
        return rdStr

    def Get_VSA_symbol_rate(self):
        rdStr = self.query(':SENS:DDEM:SRAT?')
        return rdStr

    def Get_VSA_SymbolRateError(self):
        rdStr = self.query(':CALC2:MARK:FUNC:DDEM:STAT:SRER?')
        return rdStr

    ###########################################################################
    ### VSA Init Functions
    ###########################################################################
    def Init_VSA(self,sName=""):
        self.Set_Channel("DDEM",sName)

    ###########################################################################
    ### VSA Set Functions
    ###########################################################################
    def Set_VSA_Capture_Length(self,length):
        """Number of symbols"""
        if length == 0:
            self.write(f':SENS:DDEM:RLEN:AUTO ON')
        else:
            self.write(f':SENS:DDEM:RLEN:AUTO OFF')
            self.write(f':SENS:DDEM:RLEN:VAL {length} SYM')

    def Set_VSA_Capture_Time(self,time):
        """secs"""
        if time == 0:
            self.write(f':SENS:DDEM:RLEN:AUTO ON')
        else:
            self.write(f':SENS:DDEM:RLEN:AUTO OFF')
            self.write(f':SENS:DDEM:RLEN:VAL {time} S')

    def Set_VSA_EqualizerState(self,sState):
        """ ON; OFF"""
        if sState in ('ON', 1):
            self.write(f':SENS:DDEM:EQU:STAT ON')
        elif sState in ('OFF',0):
            self.write(f':SENS:DDEM:EQU:STAT OFF')

    def Set_VSA_Filter_Alpha(self,alpha):
        """ Alpha Value """
        self.write(f':SENS:DDEM:TFIL:ALPH {alpha}')

    def Set_VSA_Filter_Type(self,sName):
        """RC | RRC | Gauss | GMSK | None """
        self.write(f':SENS:DDEM:TFIL:NAME "{sName}"')

    def Set_VSA_Mod(self,sMod):
        """ QPSK | OQPSK | 8PSK | 16APSK | 32APSK """
        if sMod == 'QPSK':
            self.Set_VSA_Mod_Type('QPSK')
            self.write(':SENS:DDEM:QPSK:FORM NORM')
        elif sMod == 'OQPSK':
            self.Set_VSA_Mod_Type('QPSK')
            self.write(':SENS:DDEM:QPSK:FORM OFFS')
        elif sMod == '8PSK':
            self.Set_VSA_Mod_Type('PSK')
            self.write(':SENS:DDEM:PSK:FORM NORM')
            self.write(':SENS:DDEM:PSK:NST 8')
        elif sMod == '16APSK':
            self.Set_VSA_Mod_Type('APSK')
            self.write(':SENS:DDEM:APSK:NST 16')
        elif sMod == '32APSK':
            self.Set_VSA_Mod_Type('APSK')
            self.write(':SENS:DDEM:APSK:NST 32')
        elif sMod == '64APSK':
            self.Set_VSA_Mod_Type('UQAM')
            self.query(':INIT:IMM;*OPC?')
            self.write(':SENS:DDEM:USER:NAME "C:\\R_S\\instr\\user\\VSA\\Constellation\\DVB-S2X\\64APSK_8_16_20_20_4_5.vam"')
        elif sMod == '256APSK':
            self.Set_VSA_Mod_Type('UQAM')
            self.query(':INIT:IMM;*OPC?')
            self.write(':SENS:DDEM:USER:NAME "C:\\R_S\\instr\\user\\VSA\\Constellation\\DVB-S2X\\256APSK_135_180.vam"')

    def Set_VSA_Mod_PSK(self,iState):
        """2 | 8"""
        self.write(f':SENS:DDEM:PSK:NST {iState}')

    def Set_VSA_Mod_QAM(self,iState):
        """16 | 32 | 64 | 128 | 256 | 512 | 1024"""
        self.write(f':SENS:DDEM:QAM:NST {iState}')

    def Set_VSA_Mod_Type(self,sMod):
        """PSK|MSK|QAM|QPSK|FSK|ASK|APSK"""
        self.write(f':SENS:DDEM:FORM {sMod}')

    def Set_VSA_Symbol_Rate(self,rate):
        """Symbol Rate, Hz"""
        self.write(f':SENS:DDEM:SRAT {rate}')

    def Set_VSA_Result_Length(self,dLength):
        """integer symbol length or 'MAX' """
        maxLen = self.Get_VSA_Capture_Length()
        if dLength == 'MAX':
            self.write(f':SENS:DDEM:TIME {maxLen}')                 # Result Length
            self.write(':CALC:ELIN:STAT ON')                        # Full Evaluation Range
        else:
            self.write(f':SENS:DDEM:TIME {int(dLength)}')           # Result Length

#####################################################################
### Run if Main
#####################################################################
if __name__ == "__main__":
    ### this won't be run when imported
    VSA = VSA()
    VSA.jav_Open("192.168.1.109")
    VSA.Init_VSA()
    VSA.Set_InitImm()
    VSA.jav_Close()



###############################################################################
### Purpose: rssd.FSW.WLAN_K54 test
###              _   ___        __  _____         _
###             | | | \ \      / / |_   _|__  ___| |_
###             | |_| |\ \ /\ / /    | |/ _ \/ __| __|
###             |  _  | \ V  V /     | |  __/\__ \ |_
###             |_| |_|  \_/\_/      |_|\___||___/\__|
###             Please connect instrument prior 2 test
###############################################################################
### User Entry
###############################################################################
host = '10.0.0.7'                                       #Get local machine name
host = '192.168.1.114'

###############################################################################
### Code Start
###############################################################################
import unittest
from rssd.VSA.WLAN_K91  import VSA

class TestGeneral(unittest.TestCase):
    def setUp(self):                                    #run before each test
        self.FSW = VSA().jav_OpenTest(host)
        self.FSW.Init_WLAN()

    def tearDown(self):                                 #Run after each test
        self.assertEqual(self.FSW.jav_Error()[0],'0')
        self.FSW.jav_Close()

###############################################################################
### <Test>
###############################################################################
    def test_FSW_WLAN_ACLR(self):
        self.FSW.Init_WLAN_ACLR()
        self.FSW.Get_ACLR()

    def test_FSW_WLAN_Get(self):
        nullVal = self.FSW.Get_WLAN_Standard()
        nullVal = self.FSW.Get_WLAN_ChBW()
        nullVal = self.FSW.Get_WLAN_PPDU()
        nullVal = self.FSW.Get_WLAN_MCS()
        nullVal = self.FSW.Get_WLAN_Modulation()
        self.assertEqual(self.FSW.jav_Error()[0],'0')

    def test_FSW_WLAN_EVM(self):
        self.FSW.Init_WLAN_EVM()
        self.FSW.Get_Params_WLAN_EVM()

    def test_FSW_WLAN_SEM(self):
        self.FSW.Init_WLAN_SEM()
        self.FSW.Get_WLAN_SEM()

    def test_FSW_WLAN_Set(self):
        # self.FSW.Set_Freq(self.Freq)
        self.FSW.Set_WLAN_ChBW(160)
        self.FSW.Set_WLAN_Standard('AC')
        self.FSW.Set_WLAN_MCS(1)
        self.FSW.Set_WLAN_Modulation('QPSK')
        self.assertEqual(self.FSW.jav_Error()[0],'0')

    def test_FSW_WLAN_Set_A(self):
        setVal = 'A'
        self.FSW.Set_WLAN_Standard(setVal)
        getVal = self.FSW.Get_WLAN_Standard()
        if self.FSW.connected: self.assertEqual(setVal,getVal)

    def test_FSW_WLAN_Set_B(self):
        setVal = 'B'
        self.FSW.Set_WLAN_Standard(setVal)
        getVal = self.FSW.Get_WLAN_Standard()
        if self.FSW.connected: self.assertEqual(setVal,getVal)

    # def test_FSW_WLAN_Set_G(self):
    #     setVal = 'G'
    #     self.FSW.Set_WLAN_Standard(setVal)
    #     getVal = self.FSW.Get_WLAN_Standard()
    #     self.assertEqual(setVal,getVal)

    def test_FSW_WLAN_Set_N(self):
        setVal = 'N'
        self.FSW.Set_WLAN_Standard(setVal)
        getVal = self.FSW.Get_WLAN_Standard()
        if self.FSW.connected: self.assertEqual(setVal,getVal)

    def test_FSW_WLAN_Set_AC(self):
        setVal = 'AC'
        self.FSW.Set_WLAN_Standard(setVal)
        getVal = self.FSW.Get_WLAN_Standard()
        if self.FSW.connected: self.assertEqual(setVal,getVal)

    def test_FSW_WLAN_Set_AX(self):
        setVal = 'AX'
        self.FSW.Set_WLAN_Standard(setVal)
        getVal = self.FSW.Get_WLAN_Standard()
        if self.FSW.connected: self.assertEqual(setVal,getVal)

    def test_FSW_WLAN_Set_Bad(self):            #Test exception
        setVal = 'BAD'
        self.FSW.Set_WLAN_Standard(setVal)

    def test_FSW_WLAN_Set_BBON(self):
        self.FSW.Set_WLAN_ChBW(20)
        if self.FSW.connected: self.FSW.Set_WLAN_BBState('ON')

###############################################################################
### </Test>
###############################################################################
if __name__ == '__main__':
    #coverage run -a -m unittest -b -v test_HW_VSA_WLAN
    suite = unittest.TestLoader().loadTestsFromTestCase(TestGeneral)
    unittest.TextTestRunner(verbosity=2).run(suite)



'''RSSD Socket Bus Class'''
# pylint: disable=E0611,E0401,E0202
# pylint: disable=signature-differs
import os
import socket
import logging
from rssd.bus.bus import bus

class jaSocket(bus):
    """Rohde & Schwarz VISA Class"""
    def __init__(self):
        self.EOL        = '\n'              # \n or \r\n
        self.K2         = 'NoVISA'
        logging.basicConfig(level=logging.INFO, \
                            filename=os.path.splitext(__file__)[0] + '.log', filemode='w', \
                            format='%(asctime)s - %(name)s - %(levelname)s - %(message)s')

    def close(self):
        """Close K2 Session"""
        try:
            self.K2.close()
        except:
            pass

    def open(self, resourceID, param):
        '''
        Open raw socket Connection
        resourceID  = IPaddress(string)
        param       = port number
        '''
        self.K2 = socket.socket()
        self.ResID = resourceID
        try:
            self.K2.settimeout(1)
            self.K2.connect((resourceID,param))
        except:
            logging.error(f'jaS_OpnErr: {self.ResID}')
            self.K2 = 'NoSOCKET'
        return self

    def query(self, SCPIstr):
        read ="<notRead>"
        try:
            cmd = SCPIstr + self.EOL
            self.K2.sendall(cmd.encode())               # Write if connected
            sOut = self.K2.recv(10000).strip()          # read socket
            read = sOut.decode()
        except:
            logging.error(f'jaS_RdErr : {self.ResID}-->{SCPIstr}')
        return read

    def timeout(self, seconds):
        self.K2.settimeout(seconds)

    def read_raw(self):
        """Read binary values from instrument"""
        sOut = self.K2.recv(10000).strip()              # read socket
        return sOut

    def write(self, SCPIstr):
        try:
            out = SCPIstr + self.EOL
            self.K2.sendall(out.encode())               #Write if connected
        except:
            logging.error(f'jaS_WrtErr: {self.ResID}-->{SCPIstr}')

    def write_raw(self, SCPIstr):
        self.write(SCPIstr)

if __name__ == "__main__":
    RS = jaSocket().open("192.168.58.115",5025)
    print(RS.query("*IDN?"))
    RS.close()



##########################################################
### Rohde & Schwarz Automation for demonstration use.
### Title  : Timing SCPI Commands Example
### Author : mclim
### Date    : 2018.05.24
##########################################################
### User Entry
##########################################################
SMW_IP      = '192.168.1.114'
FSW_IP      = '192.168.1.109'
FreqArry    = [6e9]
pwrArry     = range(-50,8,1)
LTE_Dir     = 'UL'
waveparam   =[[20,100],
              [20,66]]
subFArry    = [1]
modArry     = ['QPSK', 'QAM64']     #QPSK; QAM16; QAM64; QAM256
numMeas     = 1
AutoLvl     = 0                #0:AutoRef 1:AutoLevel
SCFDMA      = 'OFF'

##########################################################
### Code Overhead: Import and create objects
##########################################################
from datetime               import datetime     #pylint: disable=E0611,E0401
from rssd.FileIO            import FileIO       #pylint: disable=E0611,E0401
from rssd.VST.LTE           import VST          #pylint: disable=E0611,E0401
# import time
# import ctypes  # An included library with Python install
OFile = FileIO().makeFile(__file__)

##########################################################
### Code Start
##########################################################
LTE = VST().jav_Open(SMW_IP,FSW_IP,OFile)
LTE.LTE_TF     = SCFDMA
LTE.LTE_Dir    = LTE_Dir
LTE.Freq       = FreqArry[0]

##########################################################
### Measure Time
##########################################################
#sDate = datetime.now().strftime("%y%m%d-%H:%M:%S.%f") #Date String
Header = 'Iter,Model,Freq,K144Crest,K144Pwr,EVM,ChBW,SCFDMA,RB,Mod,SMWPwr,SubFram,Attn,Preamp,RefLvl,AutoLvl,AlTime,CrestF,P10_00,P01_00,P00_10,P00_01,CmdTime,TotTime'
OFile.write(Header)

LTE.FSW.Init_LTE()
LTE.FSW.Set_LTE_EVMUnit('DB')
LTE.FSW.Set_Trig1_Source('EXT')
LTE.FSW.Set_SweepCont(0)
LTE.FSW.Init_CCDF()
LTE.FSW.Set_In_YIG(0)
LTE.FSW.Set_CCDF_BW(120e6)
LTE.FSW.Set_CCDF_Samples(2e6)
LTE.FSW.Set_Trig1_Source('IMM')
LTE.FSW.Set_AttnAuto()
LTE.FSW.Set_SweepCont(0)

for i in range(numMeas):                                        #Loop: Measurements
    for mod in modArry:                                         #Loop: Modulation
        for param in waveparam:                                 #Loop: Subcarrier
            LTE.LTE_ChBW    = param[0]
            LTE.LTE_Mod     = mod
            LTE.LTE_RB      = param[1]
            for freq  in FreqArry:                              #Loop: Frequency
                LTE.Freq     = FreqArry[0]
                LTE.Set_LTE_All()                             #[[[Make Waveform]]]
                LTE.FSW.Init_CCDF()
                LTE.FSW.Set_InitImm()
                ccdf = LTE.FSW.Get_CCDF()
                print(f'Freq:{freq} RFBW:{LTE.LTE_ChBW} SubC:{LTE.LTE_RB} Mod:{LTE.LTE_Mod}')
                print(Header)
                #ctypes.windll.user32.MessageBoxW(0, "Verify", "Please Verify Waveform", 1)
                for pwr in pwrArry:                             #Loop: Power
                    LTE.SMW.Set_RFPwr(pwr)
                    tickA = datetime.now()
                    if AutoLvl == 1:
                        LTE.FSW.Init_LTE()
                        LTE.FSW.Set_Autolevel()
                    else:
                        LTE.FSW.Set_LTE_AutoRef()
                    ALTime = datetime.now() - tickA
                    for subFram in subFArry:                    #Loop: Subframe
                        LTE.FSW.Set_LTE_SweepTime((subFram)*1.1e-3)
                        LTE.FSW.Set_LTE_SubFrameCount(subFram)
                        tick = datetime.now()
                        LTE.FSW.Init_LTE()
                        LTE.FSW.Set_SweepCont(0)
                        LTE.FSW.Set_InitImm()
                        EVM = LTE.FSW.Get_LTE_EVMParams()
                        Attn = LTE.FSW.Get_AmpSettings()
                        d = datetime.now() - tick       #Measurement only
                        t = datetime.now() - tickA      #Autolevel + Measurement
                        OutStr = f'{i},{LTE.FSW.Model},{freq},{EVM},{LTE.LTE_ChBW},{LTE.LTE_TF},{LTE.LTE_RB},{LTE.LTE_Mod},{pwr:3d},{subFram},{Attn},{AutoLvl},{ALTime.seconds:3d}.{ALTime.microseconds:06d},cf:{ccdf},{d.seconds:3d}.{d.microseconds:06d},{t.seconds:3d}.{t.microseconds:06d}'
                        OFile.write(OutStr)

##########################################################
### Cleanup Automation
##########################################################
LTE.jav_Close()



##########################################################
### Rohde & Schwarz Automation for demonstration use.
###
### Title  : Timing SCPI Commands Example
### Author : mclim
### Date   : 2018.05.24
###
##########################################################
### User Entry
##########################################################
SMW_IP      = '192.168.1.114'
FSW_IP      = '192.168.1.109'
FreqArry    = [26.55e9, 28.6e9, 29.45e9, 37.05e9, 38.5e9, 39.95e9]
pwrArry     = range(-50,8,2)            #Power Array
NR_Dir      = 'UL'
waveparam   = [[100,60,128]]            #ChBW, SubSp, RB
            #   [100,120,66]]           #ChBW, SubSp, RB
            #   [200,60,264],           #ChBW, SubSp, RB
            #   [200,120,132],          #ChBW, SubSp, RB
            #   [400,120,264]]          #ChBW, SubSp, RB
subFArry    = [1]
modArry     = ['QPSK','QAM64']          #QPSK; QAM16; QAM64; QAM256; PITB
numMeas     = 1
AutoLvl     = 1                         #0:AutoEVM 1:AutoLevel
DFT_S_OFDM  = 'ON'

##########################################################
### Code Overhead: Import and create objects
##########################################################
# import time
# import ctypes                                   # An included library with Python install
from datetime               import datetime     #pylint: disable=E0611,E0401
from rssd.FileIO            import FileIO       #pylint: disable=E0611,E0401
from rssd.VST.NR5G_K144     import VST          #pylint: disable=E0611,E0401
OFile = FileIO().makeFile(__file__)

##########################################################
### Code Start
##########################################################
NR5G = VST().jav_Open(SMW_IP,FSW_IP,OFile)
NR5G.NR_TF      = DFT_S_OFDM
NR5G.NR_Dir     = NR_Dir
NR5G.Freq       = FreqArry[0]

##########################################################
### Measure Time
##########################################################
#sDate = datetime.now().strftime("%y%m%d-%H:%M:%S.%f") #Date String
Header = 'Iter,Model,Freq,K144Crest,K144Pwr,EVM,ChBW,Waveform,SubSp,Mod,SMWPwr,SubFram,Attn,Preamp,RefLvl,AutoLvl,AlTime,CrestF,P10_00,P01_00,P00_10,P00_01,CmdTime,StepTime'
OFile.write(Header)

NR5G.FSW.Init_5GNR()
NR5G.FSW.Set_5GNR_EVMUnit('DB')
NR5G.FSW.Set_Trig1_Source('EXT')
NR5G.FSW.Set_5GNR_Direction(NR_Dir)
NR5G.FSW.Set_SweepCont(0)
# NR5G.FSW.Init_CCDF()
# NR5G.FSW.Set_In_YIG(0)
# NR5G.FSW.Set_CCDF_BW(120e6)
# NR5G.FSW.Set_CCDF_Samples(2e6)
# NR5G.FSW.Set_Trig1_Source('IMM')
# NR5G.FSW.Set_AttnAuto()
NR5G.FSW.Set_SweepCont(0)

for i in range(numMeas):                                            #Loop: Measurements
    for mod in modArry:                                             #Loop: Modulation
        for param in waveparam:                                     #Loop: Waveform Parameters
            NR5G.NR_ChBW    = param[0]
            NR5G.NR_SubSp   = param[1]
            NR5G.NR_RB      = param[2]
            NR5G.NR_Mod     = mod
            for freq  in FreqArry:                                  #Loop: Frequency
                NR5G.Freq     = freq
                NR5G.Set_5GNR_All()                                 #[[[Make Waveform]]]
                # NR5G.FSW.Init_CCDF()
                # NR5G.FSW.Set_InitImm()
                # ccdf = NR5G.FSW.Get_CCDF()
                ccdf = '<tbd>,<tbd>,<tbd>,<tbd>,<tbd>'
                print(f'Freq:{freq:.0f} RFBW:{NR5G.NR_ChBW} SubC:{NR5G.NR_SubSp} Mod:{NR5G.NR_Mod}')
                print(Header)
                #ctypes.windll.user32.MessageBoxW(0, "Verify", "Please Verify Waveform", 1)
                for pwr in pwrArry:                                 #Loop: Power
                    NR5G.FSW.Init_5GNR()
                    NR5G.SMW.Set_RFPwr(pwr)
                    tickA = datetime.now()                          #TickA
                    if AutoLvl == 1:
                        NR5G.FSW.Set_Autolevel()
                    else:
                        NR5G.FSW.Set_5GNR_AutoEVM()
                    ALTime = datetime.now() - tickA
                    for subFram in subFArry:                        #Loop: Subframe
                        NR5G.FSW.Set_SweepTime((subFram)*1.1e-3)
                        NR5G.FSW.Set_5GNR_SubFrameCount(subFram)
                        tick = datetime.now()                       #Tick
                        NR5G.FSW.Init_5GNR()
                        NR5G.FSW.Set_SweepCont(0)
                        NR5G.FSW.Set_InitImm()
                        EVM  = NR5G.FSW.Get_5GNR_EVMParams()
                        Attn = NR5G.FSW.Get_AmpSettings()
                        d = datetime.now() - tick                   #Measurement only test time
                        s = datetime.now() - tickA                  #Total test time
                        OutStr = f'{i},{NR5G.FSW.Model},{freq:.0f},{EVM},{NR5G.NR_ChBW},{NR5G.NR_TF},{NR5G.NR_SubSp},{NR5G.NR_Mod},{pwr:3d},{subFram},{Attn},{AutoLvl},{ALTime.seconds:3d}.{ALTime.microseconds:06d},cf:{ccdf},{d.seconds:3d}.{d.microseconds:06d},{s.seconds:3d}.{s.microseconds:06d}'
                        OFile.write(OutStr)

##########################################################
### Cleanup Automation
##########################################################
NR5G.jav_Close()



##########################################################
### Rohde & Schwarz Automation for demonstration use.
###
### Title  : Sample SMW FSW WLAN EVM Sweep
### Author : mclim
### Date   : 2019.03.21
###
##########################################################
### User Entry
##########################################################
SMW_IP      = '192.168.1.114'
FSW_IP      = '192.168.1.109'
FreqArry    = [5.25e9,5.57e9]
pwrArry     = [-20, -10, -5]
pwrArry     = range(-50,10,3)       #Power Array
ModArry     = [['AC', 80,11],       #Std,BW,MCS
               ['AC',160,11]]       #Std,BW,MCS
SweepTime   = 0.002

##########################################################
### Code Overhead: Import and create objects
##########################################################
# import time
from datetime               import datetime     #pylint: disable=E0611,E0401
from rssd.FileIO            import FileIO       #pylint: disable=E0611,E0401
from rssd.VST.WLAN          import VST          #pylint: disable=E0611,E0401
OFile = FileIO().makeFile(__file__)

##########################################################
### Code Start
##########################################################
WLAN = VST().jav_Open(SMW_IP,FSW_IP,OFile)

##########################################################
### Measure Time
##########################################################
#sDate = datetime.now().strftime("%y%m%d-%H:%M:%S.%f") #Date String
OFile.write('Freq,Pwr,ALTime,Std,ChBW,MCS,Pwr,Attn,Preamp,RefLvl,EVM,CmdTime')

WLAN.FSW.Init_WLAN()
WLAN.FSW.Set_Trig1_Source('EXT')
WLAN.FSW.Set_SweepCont(0)

for mod in ModArry:                                             #Loop: Standard
    WLAN.WLAN_Std   = mod[0]
    WLAN.WLAN_ChBW  = mod[1]
    WLAN.WLAN_MCS   = mod[2]
    WLAN.Set_WLAN_All()                                 ### Make Waveform ###
    print(f'802.11{WLAN.WLAN_Std} RFBW:{WLAN.WLAN_ChBW} MCS:{WLAN.WLAN_MCS}')
    WLAN.FSW.Set_SweepTime(SweepTime)
#   WLAN.FSW.write(':TRAC:IQ:SRAT 160e6')

    for freq  in FreqArry:                              #Loop: Frequency
        WLAN.FSW.Set_Freq(freq)
        WLAN.SMW.Set_Freq(freq)
        WLAN.Freq = freq
        for pwr in pwrArry:                             #Loop: Power
            WLAN.SMW.Set_RFPwr(pwr)
            #Autolevel Timing
            tick = datetime.now()
            WLAN.FSW.Set_WLAN_Autolvl()
            WLAN.FSW.Set_SweepCont(0)
            ALTime = datetime.now() - tick

            # Measure EVM
            tick = datetime.now()
            WLAN.FSW.Set_InitImm()
            WLAN.FSW.Set_InitImm()
            EVM = WLAN.FSW.Get_WLAN_EVMParams()
            Attn = WLAN.FSW.Get_AmpSettings()
            d = datetime.now() - tick
            OutStr = f'{freq},{pwr:3d},{ALTime.seconds:3d}.{ALTime.microseconds:06d},{WLAN.WLAN_Std},{WLAN.WLAN_ChBW},{WLAN.WLAN_MCS},{Attn},{EVM},{d.seconds:3d}.{d.microseconds:06d}'
            OFile.write(OutStr)

##########################################################
### Cleanup Automation
##########################################################
WLAN.jav_Close()



##########################################################
### Rohde & Schwarz Automation for demonstration use.
### Title  : Sample SMW FSW WLAN EVM Sweep
### Author : mclim
### Date   : 2019.03.21
##########################################################
### User Entry
##########################################################
SMW_IP      = '192.168.1.114'
FSW_IP      = '192.168.1.109'
FreqArry    = [2.1e9,2.2e9]
pwrArry     = [-20, -10, -5]
StdArry     = ['AC','N']
CHBWArry    = [20,40]
MCSArry     = [1,3,5]
SweepTime   = 0.002

MeasEVM     = 0
MeasACLR    = 0
MeasSEM     = 1

##########################################################
### Code Overhead: Import and create objects
##########################################################
# import time
from datetime               import datetime     #pylint: disable=E0611,E0401
from rssd.FileIO            import FileIO       #pylint: disable=E0611,E0401
from rssd.VST_WLAN          import VST          #pylint: disable=E0611,E0401
OFile = FileIO().makeFile(__file__)

##########################################################
### Code Start
##########################################################
WLAN = VST().jav_Open(SMW_IP,FSW_IP,OFile)

##########################################################
### Measure Time
##########################################################
#sDate = datetime.now().strftime("%y%m%d-%H:%M:%S.%f") #Date String
OFile.write('Freq,SMWPwr,ALTime,Std,ChBW,MCS,Attn,Preamp,RefLvl,MeasPwr,EVM,SEM,TxCh,Adj-,Adj+,Alt1-,Alt1+,Alt2-,Alt2+,CmdTime')        # All

WLAN.FSW.jav_Reset()
WLAN.FSW.Init_WLAN()
WLAN.FSW.Set_Trig1_Source('EXT')
WLAN.FSW.Set_SweepCont(0)

for std in StdArry:                                             #Loop: Standard
    for chbw in CHBWArry:                                       #Loop: Ch Bandwidth
        for MCS in MCSArry:                                     #Loop: Modulation
            WLAN.WLAN_Std   = std
            WLAN.WLAN_MCS   = MCS
            WLAN.WLAN_ChBW  = chbw
            WLAN.Set_WLAN_All()                                 ### Make Waveform ###
            print(f'802.11{std} RFBW:{WLAN.WLAN_ChBW} MCS:{MCS}')
            WLAN.FSW.Set_SweepTime(SweepTime)
            WLAN.FSW.write(':TRAC:IQ:SRAT 160e6')

            for freq  in FreqArry:                              #Loop: Frequency
                WLAN.FSW.Set_Freq(freq)
                WLAN.SMW.Set_Freq(freq)
                for pwr in pwrArry:                             #Loop: Power
                    WLAN.SMW.Set_RFPwr(pwr)

                    ### Autolevel Timing
                    tick = datetime.now()
                    WLAN.FSW.write(':CONF:BURS:IQ:IMM')                             #EVM
                    WLAN.FSW.write(':SENS:DEM:FORM:BCON:AUTO 1')                    #Auto PPDU Demod
                    WLAN.FSW.Set_WLAN_Autolvl()
                    Attn = WLAN.FSW.Get_AmpSettings()
                    WLAN.FSW.Set_SweepCont(0)
                    ALTime = datetime.now() - tick
                    tick = datetime.now()
                    WLAN.FSW.Set_InitImm()

                    ### Measure EVM
                    if MeasEVM:
                        WLAN.FSW.Set_InitImm()
                        WLAN.FSW.write(':CONF:BURS:IQ:IMM')                             #EVM
                        WLAN.FSW.write(':SENS:DEM:FORM:BCON:AUTO 1')                    #Auto PPDU Demod
                        WLAN.FSW.Set_InitImm()
                        EVM = WLAN.FSW.Get_WLAN_EVMParams()
                    else:
                        EVM = '-9999,-9999'
                    ### Measure ACLR
                    if MeasACLR:
                        WLAN.FSW.write(':CONF:BURS:SPEC:ACPR:IMM')                      #Config ACLR Ch
                        WLAN.FSW.Set_InitImm()
                        WLAN.FSW.Set_InitImm()
                        ACLR = WLAN.FSW.Get_ACLR()
                    else:
                        ACLR = [-9999,-9999,-9999,-9999,-9999,-9999,-9999]

                    ### Measure SEM
                    if MeasSEM:
                        WLAN.FSW.write(':CONF:BURS:SPEC:MASK:IMM')                      #Config SEM Ch
                        WLAN.FSW.Set_InitImm()
                        SEM = WLAN.FSW.query(':CALC1:LIM:FAIL?')
                    else:
                        SEM = 'NotTested'
                    d = datetime.now() - tick
                    OutStr = f'{freq},{pwr:3d},{ALTime.seconds:3d}.{ALTime.microseconds:06d},{WLAN.WLAN_Std},{WLAN.WLAN_ChBW},{WLAN.WLAN_MCS},{Attn},{EVM},{SEM},{ACLR},{d.seconds:3d}.{d.microseconds:06d}'
                    OFile.write(OutStr)

##########################################################
### Cleanup Automation
##########################################################
WLAN.jav_Close()



"""Vector Signal Analyzer Transient Analysis Functions"""
from __future__ import print_function
from rssd.VSA.Common import VSA

class VSA(VSA):
    """ Rohde & Schwarz Vector Signal Analyzer Transient Object """
    def __init__(self):
        super(VSA, self).__init__()        #Python 2/3
        self.QRange     = 'ALL'

    #####################################################################
    ### Transient Analysis Get
    #####################################################################
    def Get_TA_HopTable(self):
        self.write(f'SENS:SIGN:MOD HOP')
        TATable = self.query('CALC:HOPD:TABL:RES? 1,3').split(',')
        numEle = 27
        print("Table size: %d"%(len(TATable)))
        for i,elem in enumerate(TATable):
            print(elem,end=' ')
            if (i % numEle) == (numEle-1):
                print('<break>\r\n')
        return 1

    def Get_TA_ChirpTable(self):
        #Manual 11.9.2
        self.write(f'SENS:SIGN:MOD CHIR')
        TATable = self.query('CALC:CHRD:TABL:RES? 1,2').split(',')
        numEle = 27
        print("Table size: %d"%(len(TATable)))
        for i,elem in enumerate(TATable):
            print(elem,end=' ')
            if (i % numEle) == (numEle-1):
                print('<break>\r\n')
        return 1

    def Get_TA_ChirpStat(self):
        #Manual 11.9.2
        """Returns large table"""
        TATable = self.queryFloatArry('CALC:CHRD:STAT:DATA?')
        return TATable

    def Get_TA_ChirpStats_TimeBegin(self):
        # Chirp Stat Col2
        #Manual 11.9.2  CURR | ALL
        rdStr = self.query(f'SENS:CHIR:TIM:BEG:AVER? {self.QRange}')
        rdStr = self.query(f'SENS:CHIR:TIM:BEG:SDEV? {self.QRange}')
        rdStr = self.query(f'SENS:CHIR:TIM:BEG:MAX? {self.QRange}')
        rdStr = self.query(f'SENS:CHIR:TIM:BEG:MIN? {self.QRange}')
        print(rdStr)
        return rdStr

    def Get_TA_ChirpStats_TimeLength(self):
        # Chirp Stat Col3
        #Manual 11.9.2  CURR | ALL
        rdStr = self.query(f'SENS:CHIR:TIM:LENG:AVER? {self.QRange}')
        rdStr = self.query(f'SENS:CHIR:TIM:LENG:SDEV? {self.QRange}')
        rdStr = self.query(f'SENS:CHIR:TIM:LENG:MAX? {self.QRange}')
        rdStr = self.query(f'SENS:CHIR:TIM:LENG:MIN? {self.QRange}')
        print(rdStr)
        return rdStr

    def Get_TA_ChirpStats_Rate(self):
        # Chirp Stat Col4
        #Manual 11.9.2  CURR | ALL
        rdStr = self.query(f'SENS:CHIR:TIM:RATE:AVER? {self.QRange}')
        rdStr = self.query(f'SENS:CHIR:TIM:RATE:SDEV? {self.QRange}')
        rdStr = self.query(f'SENS:CHIR:TIM:RATE:MAX? {self.QRange}')
        rdStr = self.query(f'SENS:CHIR:TIM:RATE:MIN? {self.QRange}')
        print(rdStr)
        return rdStr

    def Get_TA_ChirpStats_StateDev(self):
        # Chirp Stat Col5
        #Manual 11.9.2  CURR | ALL
        rdStr = self.query(f'SENS:CHIR:FREQ:CHER:AVER? {self.QRange}')
        rdStr = self.query(f'SENS:CHIR:FREQ:CHER:SDEV? {self.QRange}')
        rdStr = self.query(f'SENS:CHIR:FREQ:CHER:MAX? {self.QRange}')
        rdStr = self.query(f'SENS:CHIR:FREQ:CHER:MIN? {self.QRange}')
        print(rdStr)
        return rdStr

    def Get_TA_ChirpStats_AvgFreq(self):
        # Chirp Stat Col6
        #Manual 11.9.2  CURR | ALL
        rdStr = self.query(f'SENS:CHIR:FREQ:FREQ:AVER? {self.QRange}')
        rdStr = self.query(f'SENS:CHIR:FREQ:FREQ:SDEV? {self.QRange}')
        rdStr = self.query(f'SENS:CHIR:FREQ:FREQ:MAX? {self.QRange}')
        rdStr = self.query(f'SENS:CHIR:FREQ:FREQ:MIN? {self.QRange}')
        print(rdStr)
        return rdStr

    def Get_TA_ChirpStats_Bandwidth(self):
        # Chirp Stat Col7
        #Manual 11.9.2  CURR | ALL
        rdStr = self.query(f'SENS:CHIR:FREQ:BWID:AVER? {self.QRange}')
        rdStr = self.query(f'SENS:CHIR:FREQ:BWID:SDEV? {self.QRange}')
        rdStr = self.query(f'SENS:CHIR:FREQ:BWID:MAX? {self.QRange}')
        rdStr = self.query(f'SENS:CHIR:FREQ:BWID:MIN? {self.QRange}')
        print(rdStr)
        return rdStr

    def Get_TA_ChirpStats_FreqDevAvg(self):
        # Chirp Stat Col13
        #Manual 11.9.2  CURR | ALL
        rdStr = self.query(f'SENS:CHIR:FREQ:AVGF:AVER? {self.QRange}')
        rdStr = self.query(f'SENS:CHIR:FREQ:AVGF:SDEV? {self.QRange}')
        rdStr = self.query(f'SENS:CHIR:FREQ:AVGF:MAX? {self.QRange}')
        rdStr = self.query(f'SENS:CHIR:FREQ:AVGF:MIN? {self.QRange}')
        print(rdStr)
        return rdStr

    def Get_TA_TestStuff(self):
        TATable = self.query('CALC:CHRD:STAT:DATA?').split(',')
        print(len(TATable))
        #print(TATable)
        #self.write("MMEM:STOR6:TAB ALL,'C:\\R_S\\Instr\\asdf.data'")

    #####################################################################
    ### Transient Analysis Init
    #####################################################################
    def Init_TranAna(self):
        self.Set_Channel('TA')

    #####################################################################
    ### Transient Analysis Set
    #####################################################################
    def Set_TA_Mode(self, sMode):
        """CHIR HOP"""
        if sMode == 'HOP':
            self.write(':SENS:SIGN:MOD HOP')
        elif sMode in ('CHIR','CHIRP'):
            self.write(':SENS:SIGN:MOD CHIR')
        else:
            print('K60 Mode Not supported')

#####################################################################
### Run if Main
#####################################################################
if __name__ == "__main__":
    ### this won't be run when imported
    FSW = VSA()
    FSW.jav_Open("192.168.1.109")
    #FSW.Init_TranAna()
    FSW.Get_TA_ChirpStats_TimeBegin()
    FSW.Get_TA_ChirpStats_TimeLength()
    FSW.Get_TA_ChirpStats_Rate()
    FSW.Get_TA_ChirpStats_StateDev()
    FSW.Get_TA_ChirpStats_AvgFreq()
    FSW.Get_TA_ChirpStats_Bandwidth()
    FSW.Get_TA_ChirpStats_FreqDevAvg()
    FSW.Get_TA_TestStuff()



from __future__ import print_function
IPAddr = '192.168.1.109'
IPAddr = '169.254.2.20'
IPAddr = '10.0.0.7'

###############################################################################
### Code Start
###############################################################################
import unittest
from rssd.yaVISA import jaVisa                      #pylint:disable=E0611,E0401

class TestGeneral(unittest.TestCase):
    def setUp(self):                                #Run before each test
        print("",end="")
        self.instr = jaVisa()
        self.instr.debug = 0
        self.instr.jav_Open(IPAddr)

    def tearDown(self):                             #Run after each test
        self.instr.K2.close()

###############################################################################
### <Test>
###############################################################################
    def test_Clear(self):
        self.instr.jav_Clear()

    def test_OpenIDN(self):
        #Validates VISA Open; Query; jav_ClrErr
        self.assertNotEqual(self.instr.Make,"")

    def test_OPC_Wait(self):
        #Validates VISA Write; 
        self.instr.jav_OPC_Wait('*RST')

    def test_resourcelist(self):
        rl = self.instr.jav_reslist()
        self.assertNotEqual(rl,["No VISA"])

    def test_queryint(self):
        rd = self.instr.queryInt("*OPC?")
        self.assertEqual(rd,1)

    # def test_queryintarry(self):
    #     rd = self.instr.queryIntArry("*OPT?")
    #     self.assertEqual(rd,1)

###############################################################################
### </Test>
###############################################################################
if __name__ == '__main__':                                  # pragma: no cover
    suite = unittest.TestLoader().loadTestsFromTestCase(TestGeneral)
    unittest.TextTestRunner(verbosity=2).run(suite)



"""RSSD utility"""
import argparse
import sys
import os
import rssd
from rssd.yaVISA_socket import jaVisa

def main():
    parser = argparse.ArgumentParser(description='RSSD Helper')
    parser.add_argument('-i','--idn', required=False, help='IDN from ip address')
    parser.add_argument('-b','--bar', required=False, help='Description for bar argument')
    args = parser.parse_args()          #Dictionary of args

    print(f'RSSD Examples@ {os.path.dirname(rssd.__file__)}\\examples')
    os.chdir(os.path.dirname(rssd.__file__)+'\\examples')

    if args.idn:
        try:
            instr = jaVisa()
            instr.jav_Open(args.idn)
            instr.jav_Close()
        except:
            print(f'Could not open {args.idn}')

    if args.bar:
        print(f'BBar is: {args.bar}')

    # print(f'Number of arguments: {len(sys.argv)} arguments.')
    # print(f'Argument List:{str(sys.argv)}')

if __name__ == "__main__":
    main()
    sys.exit(0)



"""Vector Signal Generator Fading Methods
"""
from rssd.VSG.Common import VSG             #pylint: disable=E0611,E0401

class VSG(VSG):                             #pylint: disable=E0102
    """ Rohde & Schwarz Vector Signal Generator 5GNR Object """
    def __init__(self):
        super(VSG,self).__init__()          #Python2/3
        self.Model = "SMW"
        self.entity = 1

    #####################################################################
    ### Fading Get Methods
    #####################################################################
    def Get_Fade_Standard(self):
        if self.Get_SMW_Mode() == 'ADV':
            rdStr = self.queryInt(f'ENT{self.entity}:SOUR1:FSIM:STAN?')
        else:
            rdStr = 0
        return rdStr

    def Get_Fade_State(self):
        if self.Get_SMW_Mode() == 'ADV':
            rdStr = self.queryInt(f'ENT{self.entity}:SOUR1:FSIM:STAT?')
        else:
            rdStr = 0
        return rdStr

    def Get_SMW_Mode(self):
        rdStr = self.queryInt(f'SCON:MODE?')
        return rdStr

    #####################################################################
    ### Fading Set Methods
    #####################################################################
    def Set_Fade_State(self,State):
        """ON OFF 1 0 """
        self.K2.timeout = 20000
        if State in (1, 'ON'):
            self.write(':SCON:MODE ADV')
            self.write(':SCON:APPL;*OPC?')
            self.delay(1)
            self.jav_OPC_Wait(f'ENT{self.entity}:SOUR1:FSIM:STAT 1')
        elif State in (0, 'OFF'):
            # self.write(':SCON:MODE STAN')
            # self.write(':SCON:APPL')
            self.jav_OPC_Wait(f'ENT{self.entity}:SOUR1:FSIM:STAT 0')
        else:
            print('State not supported, please set ON or OFF')
        self.K2.timeout = 5000


#####################################################################
### Run if Main
#####################################################################
if __name__ == "__main__":
    # this won't be run when imported
    SMW = VSG()
    SMW.jav_Open("192.168.1.115")
    SMW.entity = 2
    SMW.Set_Fade_State(1)
    SMW.jav_Close()



"""rssd.VSA.ADemod_K7 test"""
host = '192.168.1.109'                              #Get local machine name

###############################################################################
### Code Start
###############################################################################
import unittest
from rssd.VSA.ADemod_K7     import VSA              #pylint: disable=E0611,E0401

class TestGeneral(unittest.TestCase):
    def setUp(self):                                #run before each test
        self.FSW = VSA().jav_OpenTest(host)
        self.FSW.Init_ADemod()

    def tearDown(self):                             #Run after each test
        self.assertEqual(self.FSW.jav_Error()[0],'0')
        self.FSW.jav_Close()

###############################################################################
### <Test>
###############################################################################
    def test_FSW_ADemod(self):
        self.FSW.Set_Adem_dbw(3e6)
        self.FSW.Set_Adem_LPassStat('OFF')
        self.FSW.Set_Adem_LPassStat('ON')
        self.FSW.Set_Adem_LPassAbsolute('3kHz')
        self.FSW.Set_Adem_LPassManual(1e6)
        self.FSW.Set_Adem_LPassRelative('5PCT')
        getVal = self.FSW.Get_Adem_dbw()
        if self.FSW.connected: self.assertEqual(getVal, 3e6)

    def test_FSW_ADemod_PM(self):
        self.FSW.Set_Freq(28e9)                          # RF Freq
        self.FSW.Set_Adem_dbw(200e3)                # Demod BW
        self.FSW.Set_Adem_Coupling('AC')            # Coupling
        self.FSW.Set_Adem_PM_Unit('DEG')            # Units
        self.FSW.Set_Adem_PM_Scale(4)               # Y Scaling
        self.FSW.Set_Adem_PM_RefPos(50)             # Phase Reference Position
        self.FSW.Set_Adem_PM_RefVal(1)              # Phase Reference Value
        self.FSW.Set_In_YIG('ON')                        # YIG ON

###############################################################################
### </Test>
###############################################################################
if __name__ == '__main__':
    #coverage run -a -m unittest -b -v test_HW_VSA_ADemod
    suite = unittest.TestLoader().loadTestsFromTestCase(TestGeneral)
    unittest.TextTestRunner(verbosity=2).run(suite)



"""Sweep FSW/SMW Frequncy"""
##########################################################
### User Entry
##########################################################
SMW_IP = '192.168.1.115'                    #IP Address
FSW_IP = '192.168.1.109'                    #IP Address
FreqStart = int(24e9)
FreqStop = int(40e9)
FreqStep = int(1e9)
fSpan = 100e6
SWM_Out = -20
Mixer = 0

##########################################################
### Code Start
##########################################################
import time
from rssd.FSW_Common    import VSA
from rssd.SMW_Common    import VSG
from rssd.FileIO        import FileIO

OFileCSV = FileIO().makeFile(__file__+'csv')
OFileXML = FileIO().makeFile(__file__+'xml')
SMW = VSG().jav_Open(SMW_IP,OFileCSV)  #Create SMW Object
FSW = VSA().jav_Open(FSW_IP,OFileCSV)  #Create FSW Object

##########################################################
### Instrument Settings
##########################################################
SMW.Set_RFPwr(SWM_Out)                    #Output Power
SMW.Set_RFState('ON')                     #Turn RF Output on

FSW.Set_SweepCont(0)
#FSW.Set_SweepTime(200e-3)
FSW.Set_Span(fSpan)

if Mixer:                                 #Mixer
    FSW.write('SENS:MIX:STAT ON')
    FSW.write('SENS:MIX:HARM:BAND V')

for freq in range(FreqStart,FreqStop,FreqStep):
    SMW.Set_Freq(freq)
    time.sleep(0.01)
    FSW.Set_Freq(freq)
    FSW.Set_InitImm()
    FSW.Set_Mkr_Peak()
    Mkr = FSW.Get_Mkr_XY()
    OFileCSV.write(f'{freq},{Mkr[0]},{Mkr[1]}')
    OFileXML.write(f'  <Point x="{Mkr[0]}" y="{Mkr[1]}"/>')

SMW.jav_ClrErr()                          #Clear Errors
FSW.jav_ClrErr()                          #Clear Errors



"""Timing SCPI Commands Example"""
import timeit
from rssd.instrument        import instr

# K2SO = instr().open('192.168.58.114')
K2SO = instr().open('192.168.58.114', type='hislip')
K2SO.timeout(1)

def timeQuery(SCPI):
    tick = timeit.default_timer()
    rdStr = K2SO.query(SCPI)
    TotTime = timeit.default_timer() - tick
    outStr = f'{TotTime:.3f},{rdStr}'
    return outStr

# data = timeQuery(':INIT:IMM;*OPC?')
data = timeQuery(':CAL1:IQM:LOC?')
# capture = K2SO.query(':SENS:SWE:TIME?')
print(f'{data}')

K2SO.close()



""" Vector Signal Analyzer Common Functions"""
### Author:  Martin C Lim
### Date:    2018.02.01
#####################################################################
from rssd.yaVISA import jaVisa              # pylint: disable=E0611,E0401
try:
    import rssd.VSA_Leveling as VSAL        # pylint: disable=E0611,E0401
except:
    pass

class VSA(jaVisa):
    """ Rohde & Schwarz Vector Signal Analyzer Object """
    def __init__(self):
        super(VSA, self).__init__()
        self.Model  = "FSW"
        self.CurrCh = ""

    #####################################################################
    ### FSW Get
    #####################################################################
    def Get_ACLR(self):
        ACLR = self.query(':CALC:MARK:FUNC:POW:RES? MCAC')
        return ACLR

    def Get_Mkr_BandACLR(self):
        for i in range(1, 3+1):
            if i == 1:
                ACLR = f'{self.Get_Mkr_Band(i)[1]:7.3f}'
            else:
                ACLR = f'{ACLR},{self.Get_Mkr_Band(i)[1]:7.3f}'
        return ACLR

    def Get_AttnMech(self):
        out = self.queryInt('INP:ATT?')
        return out

    def Get_CCDF(self):
        P10_00 = self.queryFloat(f'CALC:STAT:CCDF:X1? P10;*WAI')
        P01_00 = self.queryFloat(f'CALC:STAT:CCDF:X1? P1;*WAI')
        P00_10 = self.queryFloat(f'CALC:STAT:CCDF:X1? P0_1;*WAI')
        P00_01 = self.queryFloat(f'CALC:STAT:CCDF:X1? P0_01;*WAI')
        CrestF = self.queryFloat(f'CALC:STAT:RES? CFAC')
        return f'{CrestF:.4f},{P10_00:.2f},{P01_00:.2f},{P00_10:.2f},{P00_01:.2f}'

    def Get_ChPwr(self):
        out = self.queryFloat('FETC:SUMM:POW?')
        return out

    def Get_ChannelName(self):
        ChList  = self.Get_Channels()
        CurrApp = self.query('INST?')
        match   = [x for x in ChList if CurrApp in x]
        index   = ChList.index(match[0])
        self.CurrCh  = ChList[index+1]
        return self.CurrCh

    def Get_Channels(self):
        ChList  = self.query('INST:LIST?').replace("\'","").split(',')
        return ChList

    def Get_EVM(self):
        #EVM = self.query('FETC:SUMM:EVM:ALL:AVER?')
        out = self.queryFloat('FETC:SUMM:EVM?;*WAI')
        return out

    def Get_Freq(self):
        rdStr = self.queryFloat(':SENS:FREQ:CENT?')
        return rdStr

    def Get_Harm(self):
        rdStr = self.queryFloatArry(':CALC:MARK:FUNC:HARM:LIST?')
        return rdStr

    def Get_IFOvld(self):
        self.Set_InitImm()
        rdStr = self.query("STAT:QUES:POW:COND?").strip()
        return rdStr

    def Get_IQ_Data(self,sFilename="file.iqw"):
        """ IQ data --> IQW file for VSE"""
        self.write("FORM REAL,32")
        self.write("TRAC:IQ:DATA:FORM IQP")
        self.write("TRAC:IQ:DATA?")
        data = self.jav_read_raw()

        samples = self.Get_IQ_RecLength()

        # Read num of digits to get for No of floats
        if int(samples) < 125000000:
            digits = data[1]
        else:
            digits = "10"

        # # Don't need this but including for completeness
        # # Reads total number of bytes that holds IQ data

        # i = 2
        # totalbytes = ""
        # while i <= int(digits)+1:
        #   totalbytes = totalbytes + data [i]
        #   i += 1

        iqfile = open(sFilename, "wb")
        iqfile.write(data[2 + int(digits):])
        iqfile.close()

    def Get_IQ_Data_Ascii(self,MLEN=1e3):
        CSVd = ""
        self.write('Format:DATA ASCII')
        self.write('TRAC:IQ:DATA:FORM IQP')
        RLEN = self.Get_IQ_RecLength()                          #Sweep Points
        numLoops  = int(round(RLEN/MLEN))+1
        for i in range(numLoops):                               # pylint: disable=E0602
            SCPI = "TRAC:IQ:DATA:MEM? %d,%d"%((i * MLEN),MLEN)  #TRAC:IQ:DATA:MEM? <MemStrt>,<MLEN>
            CSVd = CSVd + self.query(SCPI)                      #IQ Dump
        print("Memory Done Reading %d"%len(CSVd.split(',')))
        return CSVd

    def Get_IQ_Data_Ascii2(self):
        """Return IQ data as CSV string"""
        CSVd = ""
        self.write('FORMAT:DATA ASCII')
        self.write('TRAC:IQ:DATA:FORM IQP')
        CSVd = self.query("TRAC:IQ:DATA:MEM?")
        #print("Memory Done Reading %d"%len(CSVd.split(',')))
        return CSVd

    def Get_IQ_Data_Bin(self):
        import struct                                   #pylint: disable=C0415
        self.write('FORMAT:DATA REAL,32')
        self.write('TRAC:IQ:DATA:FORM IQP')
        self.write('TRAC:IQ:DATA:MEM?')
        rdStr = self.jav_read_raw()
        numBytes = int(chr(rdStr[1]))                   # Number of Bytes
        numIQ    = int(rdStr[2:2+numBytes])
        IQBytes  = rdStr[(numBytes+2):-1]               # Remove Header
        IQAscii  = struct.unpack("<" + 'f' * int(numIQ/4),IQBytes)
        print(IQAscii[0:10])
        return IQBytes

    def Get_IQ_RecLength(self):
        RLEN = self.queryInt('TRAC:IQ:RLEN?')	        #Record(Samples) Length
        return RLEN

    def Get_IQ_SamplingRate(self):
        # SamplingRate = IQ_BW / 0.8
        rdStr = self.queryFloat('TRAC:IQ:SRAT?')        #Sampling Rate
        return rdStr

    def Get_Mkr_Band(self,iNum=1,iWind=1):
        ValX = self.queryFloat(':CALC%d:MARK%d:X?'%(iWind,iNum))
        ValY = self.queryFloat(':CALC%d:MARK%d:FUNC:BPOW:RES?'%(iWind,iNum))
        return [ValX, ValY]

    def Get_Mkr_Freq(self,iNum=1,iWind=1):
        MkrFreq = self.queryFloat(':CALC%d:MARK%d:X?'%(iWind,iNum))
        return float(MkrFreq)

    def Get_Mkr_Noise(self,iNum=1,iWind=1):
        ValX = self.queryFloat(':CALC%d:MARK%d:X?'%(iWind,iNum))
        ValY = self.queryFloat(':CALC%d:MARK%d:FUNC:NOIS:RES?'%(iWind,iNum))
        return [ValX, ValY]

    def Get_Mkr_TimeDomain(self,iNum=1,iWind=1):
        # self.write(':CALC:MARK%d:FUNC:SUMM:STAT ON'%iNum)
        #MkrFreq = self.query(':CALC%d:MARK%d:X?'%(iWind,iNum)).strip()
        MkrPwr  = self.queryFloat(f':CALC{iWind}:MARK{iNum}:FUNC:SUMM:RMS:RES?')
        return MkrPwr

    def Get_Mkr_XY(self,iNum=1,iWind=1):
        ValX = self.queryFloat(':CALC%d:MARK%d:X?'%(iWind,iNum))
        ValY = self.queryFloat(':CALC%d:MARK%d:Y?'%(iWind,iNum))
        return [ValX, ValY]

    def Get_Mkr_Y(self,iNum=1,iWind=1):
        ValY = self.queryFloat(':CALC%d:MARK%d:Y?'%(iWind,iNum))
        return ValY

    def Get_Ovld_Stat(self):
        self.Set_InitImm()
        Read = self.queryInt('STAT:QUES:POW:COND?')
        RF_Ovld = Read & 1      # pylint: disable=W0612
        RF_Udld = Read & 2      # pylint: disable=W0612
        IF_Ovld = Read & 4      # pylint: disable=W0612
        return Read

    def Get_Params(self,amp,swp,sys,trc,header=0):                      #pylint: disable=R0913
        """Amp,Sweep,System,Trace"""
        outStr = ""
        outStr += self.Get_Params_Amp(header)+","   if (amp==1) else ""
        outStr += self.Get_Params_Sweep(header)+"," if (swp==1) else ""
        outStr += self.Get_Params_System(header)+","if (sys==1) else ""
        outStr += self.Get_Params_Trace(header)+"," if (trc==1) else ""
        return outStr[0:-1]

    def Get_Params_Amp(self,header=0):
        """Retrieve Parameters for test logs"""
        if header != 1:
            attn    = self.Get_AttnMech()
            prea    = self.Get_Preamp()
            refl    = self.Get_RefLevel()
            outStr  = f'{attn:2d},{prea},{refl:7.3f}'
        else:
            outStr = 'Attn,PreAmp,RefLvl'
        return outStr

    def Get_Params_MkrBand(self,header=0):
        """Retrieve Parameters for test logs"""
        if header != 1:
            BndMkr  = self.Get_Mkr_Band()
            outStr  = f'{BndMkr[0]:.0f},{BndMkr[1]:.3f}'
        else:
            outStr = 'MkrFreq,MkrBndPwr'
        return outStr

    def Get_Params_Mkr(self,header=0):
        """Retrieve Parameters for test logs"""
        if header != 1:
            BndMkr  = self.Get_Mkr_XY()
            outStr  = f'{BndMkr[0]:.0f},{BndMkr[1]:.3f}'
        else:
            outStr = 'MkrFreq,MkrPwr'
        return outStr

    # def Get_Params_EVM(self):
    #     MAttn   = self.Get_AttnMech()
    #     RefLvl  = self.Get_RefLevel()
    #     Power   = self.Get_ChPwr()
    #     EVM     = self.Get_EVM()
    #     return f"{MAttn:.2f},{RefLvl:.2f},{Power:6.2f},{EVM:.2f}"

    def Get_Params_Sweep(self,header=0):
        # SwpTime,SwpPts,SwpType,SwpOpt,
        if header != 1:
            Time    = self.Get_SweepTime()
            Points  = self.Get_SweepPoints()
            Type    = self.Get_SweepType()
            Opt     = self.Get_SweepOpt()
            outStr  = f'{Time:5.3f},{Points},{Type},{Opt}'
        else:
            outStr  = 'SwpTimeM,SwpPts,SwpType,SwpOpt'
        return outStr

    def Get_Params_System(self,header=0):
        if header != 1:
            error  = self.jav_Error()
            ext    = self.Get_System_ErrorExt().replace('"','')
            outStr = f'{error[0]:>4},{error[1]:10.10},{ext:10.10}'
        else:
            outStr  = 'ErrNo,ErrMsg,ExtError'
        return outStr

    def Get_Params_Trace(self,header=0,trace=1):
        if header != 1:
            mode    = self.Get_Trace_Mode(trace)
            detect  = self.Get_Trace_Detector(trace)
            avgtype = self.Get_Trace_AvgType()
            outStr  = f'{mode},{detect},{avgtype}'
        else:
            outStr  = 'TrcMode,TrcDet,AvgMode'
        return outStr

    def Get_Preamp(self):
        return self.queryInt(f'INP:GAIN:STAT?')

    def Get_RefLevel(self):
        RefLvl = self.queryFloat('DISP:TRAC:Y:RLEV?')
        return RefLvl

    def Get_Screenshot(self,file='screenshot'):
        ### File will be in FSW's C:\R_S\Instr\User
        self.write(f'MMEM:NAME "C:\\R_S\\INSTR\\USER\\{file}"')
        self.write('HCOP:CONT WIND')     #Print Displayed Windo
        self.write('HCOP:CMAP:DEF4')     #Screeen Colors
        self.write('HCOP:DEST "MMEM"')   #Send Data to file
        self.write('HCOP:DEV:LANG JPG')  #Save JPG
        self.write('HCOP:IMM')           #Create File

    def Get_SweepOpt(self):
        """AUTO | SPEed | DYN"""
        rdStr = self.query(':SENS:SWE:OPT?')
        return rdStr

    def Get_SweepPoints(self):
        rdStr = self.queryInt(':SENS:SWE:POIN?')                 #Number of trace points
        return rdStr

    def Get_SweepTime(self):
        rdStr = self.queryFloat('SENS:SWE:TIME?')                 #Sweep/Capture Time
        return rdStr

    def Get_SweepType(self):
        """AUTO | SWE | FFT"""
        rdStr = self.query(':SENS:SWE:TYPE?')
        return rdStr

    def Get_System_ErrorExt(self):
        rdStr = self.query(':SYST:ERR:EXT? ALL')
        return rdStr

    def Get_Trace_Data(self,trace=1):
        self.write('FORM ASCII ')
        DataY = self.query(f'TRAC{trace}:DATA? TRACE1')
        DataX = self.query(f'TRAC{trace}:DATA:X? TRACE1')
        return [DataX.split(','),DataY.split(',')]

    def Get_Trace_Detector(self,trace=1):
        rdStr = self.query(f'SENS:WIND1:DET{trace}?')
        return rdStr

    def Get_Trace_Mode(self,trace=1):
        rdStr = self.query(f'DISP:TRAC{trace}:MODE?')
        return rdStr

    def Get_Trace_AvgType(self):
        rdStr   = self.query('SENS:AVER:TYPE?')
        return rdStr

    #####################################################################
    ### Measurement Init
    #####################################################################
    def Init_ACLR(self, sName=""):
        self.Set_Channel("SAN",sName)
        # self.Set_ChannelName("Spectrum",sName)
        self.write('CALC:MARK:FUNC:POW:SEL ACP')

    def Init_CCDF(self, sName=""):
        self.Set_Channel("Spectrum",sName)
        self.write('CALC:STAT:CCDF ON;*WAI')

    def Init_Harm(self, sName=""):
        self.Set_Channel("Spectrum",sName)
        self.write('CALC:MARK:FUNC:HARM ON')

    def Init_IQ(self, sName=""):
        self.Set_Channel("IQ",sName)

    def Init_Spectral(self, sName=""):
        self.Set_Channel("SAN",sName)
        self.write(':SENS:FREQ:MODE SWE')

    #####################################################################
    ### Set Methods
    #####################################################################
    #####################################################################
    ### FSW ACLR
    #####################################################################
    def Set_ACLR_AdjBW(self,dCHBW):
        self.write(f'POW:ACH:BAND:ACH {dCHBW};ALT1 {dCHBW};ALT2 {dCHBW}')

    def Set_ACLR_AdjSpace(self,dCHBW):
        self.write(f'POW:ACH:SPAC:ACH {dCHBW};ALT1 {2*dCHBW};ALT2 {3*dCHBW}')

    def Set_ACLR_CHBW(self,dCHBW):
        self.write('POW:ACH:BAND %d'%dCHBW)

    def Set_ACLR_NumAdj(self,iAdj):
        self.write(f'POW:ACH:ACP {iAdj}')                           #two adjacent channels

    def Set_AttnMech(self,fMAttn):
        #self.write('INP:EATT:STAT OFF')
        self.write('INP:ATT %.0f'%fMAttn)

    def Set_AttnAuto(self):
        self.write(':INP:ATT:AUTO ON')

    def Set_Autolevel(self):
        self.query(':SENS:ADJ:LEV;*OPC?')
        # self.jav_OPC_Wait(':SENS:ADJ:LEV')

    # def Set_Autolevel_Proto(self,sState):
    # ### Used by WLAN; K96.  Please use ADJ:LEV;
    #     self.write('CONF:POW:AUTO %s;*WAI'%sState)      #ON|OFF|1|0

    def Set_Autolevel_IFOvld(self):
        ####################################################################
        # """ Algorithm designed by Darren Tipton, RSUK"""
        # """ Optimise level for Mixer Input => Optimal EVM """
        # """ Optimises for signals using IF gain as well as 1dB steps """
        ####################################################################
        optmix = 10                         # Optimal mixer level
        self.Set_SweepCont(0)
        self.Set_Autolevel()
        level = self.Get_Mkr_TimeDomain()

        # """ Switch Pre-Amp """
        if level >= -20:
            self.query("INP:GAIN:STAT OFF; *OPC?")
            gain = 0
            maxmix = 0
        else:
            self.query("INP:GAIN:STAT ON; *OPC?")
            gain = 20
            maxmix = -30

        rfatt = level + gain - optmix       #Calc RfAttn for optimal mixer level
        if rfatt < 0: rfatt = 0             #If calculated RF atten < 0, set 0
        self.Set_AttnMech(rfatt)            #Set Attenuation

        reflev = maxmix + rfatt
        self.Set_RefLevel(reflev)           #Set RefLevel

        ifovl = self.Get_Ovld_Stat()        #Check Overload
        print("Inital: Ovl:%d Attn:%d RfLvl:%d"%(ifovl,rfatt,reflev))

        # """ Optimising for attenuation """
        while ifovl != 0:
            print(f"Atnloop: Ovl:{ifovl} Attn:{rfatt} RfLvl:{reflev}")
            rfatt = rfatt + 1
            self.Set_AttnMech(rfatt)

            reflev = maxmix + rfatt
            self.Set_RefLevel(reflev)
            ifovl = self.Get_Ovld_Stat()    #Check Overload
            if reflev > 30:
                break

        # """ Optimising for reference level """
        while reflev > (-20 - gain) and ifovl == 0:
            print(f"Refloop: Ovl:{ifovl} Attn:{rfatt} RfLvl:{reflev}")
            reflev = reflev - 1
            self.Set_RefLevel(reflev)
            ifovl = self.Get_Ovld_Stat()    #Check Overload
            if reflev > 30:
                break

        # """ Final check for IF Overload """
        if ifovl != 0:
            reflev = reflev + 1
            self.Set_RefLevel(reflev)
        print(f"Final : Ovl:{ifovl} Attn:{rfatt} RfLvl:{reflev}")

    def Set_Autolevel_IQIF(self,tables):
        VSAL.Optimise_FSx_Level(self,tables)

    #####################################################################
    ### FSW CCDF
    #####################################################################
    def Set_CCDF(self,sState):
        self.write(f'CALC:STAT:CCDF {sState} ;*WAI')        #ON|OFF|1|0

    def Set_CCDF_BW(self,BW):
        self.Set_ResBW(BW)

    def Set_CCDF_Samples(self,iSamples):
        self.write(f'CALC:STAT:NSAM {iSamples}')

    def Set_Channel(self,Chan,sName=""):
        """ SAN, IQ, NR5G, LTE, WLAN, PNOISE, NOISE, SPUR, ADEM, DDEM, V5GT, AMPL """
        if sName == "":
            sName = Chan
        ChList = self.query('INST:LIST?').split(',')
        #print("Chan:%s in %s"%(Chan,ChList))
        if f"'{sName}'" in ChList:
            pass
        else:
            self.query(":INST:CRE %s,'%s';*OPC?"%(Chan,sName))
        self.query(":INST:SEL '%s';*OPC?"%sName)

    def Set_ChannelName(self,newName,oldName):
        self.write(f":INST:REN '{newName}', '{oldName}'")

    def Set_ChannelSelect(self,sName):
        self.query(":INST:SEL '%s';*OPC?"%sName)

    def Set_DisplayUpdate(self,state):
        self.write('SYST:DISP:UPD %s'%state)      #Display Update State

    #####################################################################
    ### FSW Equalization K544
    #####################################################################
    def Set_EQ_File(self,sFile):
        #self.write('SENS:CORR:FRES:Input1:USER:SLIS1:SEL "c:\\R_S\\Instr\\Debug\\K544\\IFH.s2p"')
        self.write('SENS:CORR:FRES:Input1:USER:SLIS1:SEL "%s"'%sFile)

    def Set_EQ_State(self,sState):
        """ON | OFF"""
        self.write('SENS:CORR:FRES:Input1:USER:PRES')
        self.write(f'SENS:CORR:FRES:Input1:USER:STATe {sState}')

    def Set_Freq(self,fFreq):
        """Hz"""
        self.write(':SENS:FREQ:CENT %.0f HZ'%fFreq)

    def Set_FreqStart(self,fFreq):
        """Hz"""
        self.write(':SENS:FREQ:STAR %f'%fFreq)

    # def Set_FreqStep(self,fFreq):
    #     """Hz"""
    #     self.write(':SENS:FREQ:STEP %f'%fFreq)

    def Set_FreqStop(self,fFreq):
        """Hz"""
        self.write(':SENS:FREQ:STOP %f'%fFreq)

    def Set_Harm_num(self, num):
        self.write(f':CALC1:MARK1:FUNC:HARM:NHAR {num}')

    def Set_Harm_adjust(self):
        """ Adjusts Ref Lvl, Attn, SwpTime """
        self.write(':CALC1:MARK1:FUNC:HARM:PRES')

    #####################################################################
    ### FSW IQ Analyzer
    #####################################################################
    def Set_IQ_ACLR(self, ChBW, ChSpace):
        ## Author: Darren Tipton, RSUK
        Freq = self.Get_Freq()

        #Configure Advanced FFT parameters
        self.Set_IQ_Adv_Mode()
        self.Set_IQ_Adv_TransAlgo('AVER')
        self.Set_IQ_Adv_FFTLenth(16384)                     #Reduce RBW
        self.Set_IQ_Adv_Window('BLAC')
        self.Set_IQ_Adv_WindowLenth(16384)                  #Reduce RBW

        self.Set_IQ_BW(3.1*ChSpace)
        self.Set_IQ_SpectrumWindow()                        # Add Spectrum Trace
        self.Set_Trace_Detector('RMS')                      # RMS detector
        self.Set_Mkr_Freq(Freq,1)                           # Tx Freq
        self.Set_Mkr_Band(ChBW,1)                           # Tx RFBW
        self.Set_Mkr_Freq(Freq-ChSpace,2)                   # Adj- Freq
        self.Set_Mkr_Band(ChBW,2)                           # Adj- RFBW
        self.Set_Mkr_Freq(Freq+ChSpace,3)                   # Adj+ Freq
        self.Set_Mkr_Band(ChBW,3)                           # Adj+ RFB
        self.Set_SweepCont(0)

    def Set_IQ_Adv_FFTLenth(self, dLength):
        self.write(f'IQ:FFT:LENG {dLength}')

    def Set_IQ_Adv_Mode(self, State=1):
        """Turn Advanced Mode on"""
        if (State == 'ON') or (State == 1):
            self.write("IQ:BAND:MODE FFT")
        elif (State == 'OFF') or (State == 0):
            self.write("IQ:BAND:MODE AUTO")
        else:
            print('State not supported, please set ON or OFF')

    def Set_IQ_Adv_TransAlgo(self, sInput):
        """ AVER SING """
        self.write(f"IQ:FFT:ALG {sInput}")

    def Set_IQ_Adv_Window(self, sInput):
        """ BLAC FLAT GAUS RECT P5 """
        self.write(f'IQ:FFT:WIND:TYPE {sInput}')

    def Set_IQ_Adv_WindowLenth(self, dLength):
        self.write(f"IQ:FFT:WIND:LENG {dLength}")

    def Set_IQ_BW(self,fFreq):
        # IQ_BW = SamplingRate * 0.8
        self.write('TRAC:IQ:BWID %f'%fFreq)         # Analysis BW

    def Set_IQ_RecLength(self,iLen):
        self.query('TRAC:IQ:RLEN %d'%iLen)          # Record(Samples) Length

    def Set_IQ_Samples(self,iNum):
        # Samples = MeasTime * SamplingRate
        self.write('TRAC:IQ:RLEN %d'%iNum)          # Samples

    def Set_IQ_SamplingRate(self,fFreq):
        # SamplingRate = IQ_BW / 0.8
        self.write('TRAC:IQ:SRAT %f'%fFreq)         # Sampling Rate

    def Set_IQ_SpectrumWindow(self):
        # windList = self.query('LAY:CAT:WIND?').split(',')
        # numWind = len(windList)
        # if numWind > 2:
        #     for indx in range(2,int(numWind/2)+1):
        #         self.write(f'LAY:REM "{indx}"')
        # self.write(":LAY:ADD:WIND? '1',RIGH,FREQ")
        # self.write(":DISP:WIND2:SUBW:SEL")
        self.write("LAY:REPL '1',Freq")

    def Set_IQ_Time(self,fSwpTime):
        self.Set_SweepTime(fSwpTime)

    def Set_IQ_WideBandMax(self,fFreq):
        """80 160 320 500 1200 MHz"""
        self.write('TRAC:IQ:WBAN:STAT ON')                  #Wideband reduction activated
        self.write('TRAC:IQ:WBAN:MBW %f; *WAI'%fFreq)

    def Set_In_HPFilter(self,sState):                       #Filter for 1-3GHz meas
        """0 1 ON OFF"""
        self.write('INP:FILT:HPASs:STATe %s'%sState)

    def Set_In_YIG(self,sState):
        """0 1 ON OFF"""
        self.write('INP:FILT:YIG:STATe %s'%sState)

    def Set_InitImm(self):
        self.query('INIT:IMM;*OPC?')

    def Set_Input(self,sType):
        """ RF|AIQ|DIQ|FILE """
        self.write('INP:SEL %s'%sType)

    #####################################################################
    ### FSW marker
    #####################################################################
    def Set_Mkr_AllOff(self,iWind=1):
        self.write(':CALC%d:MARK:AOFF'%(iWind))

    def Set_Mkr_Band(self,fFreq,iNum=1,iWind=1):
        self.write(f':CALC{iWind}:MARK{iNum}:FUNC:BPOW:STAT ON')
        self.write(f':CALC{iWind}:MARK{iNum}:FUNC:BPOW:SPAN {fFreq}')

    def Set_Mkr_BandDelta(self,fFreq,iNum=1,iWind=1):
        self.write(f':CALC{iWind}:DELT{iNum}:STAT ON')
        self.write(f':CALC{iWind}:DELT{iNum}:FUNC:BPOW:STAT ON')
        self.write(f':CALC{iWind}:DELT{iNum}:FUNC:BPOW:SPAN {fFreq}')
        self.write(f':CALC{iWind}:DELT{iNum}:FUNC:BPOW:MODE RPOW')

    def Set_Mkr_BandSetRef(self):
        """Autolevel w/ BandMarkers"""
        self.Set_AttnAuto()
        self.Set_SweepCont(0)
        self.Set_InitImm()                                   # Take Sweep
        ChPwr = self.Get_Mkr_Band(1)[1]
        self.Set_RefLevel(ChPwr + 5)
        if ChPwr > -24:         #FSVA:-24
            self.Set_Preamp(0)
        else:
            self.Set_Preamp(1)
            self.write('INP:GAIN:VAL 15')

    def Set_Mkr_Freq(self,fFreq,iNum=1,iWind=1):
        self.write(':CALC%d:MARK%d:X %fHz'%(iWind,iNum,fFreq))

    def Set_Mkr_Next(self,iNum=1,iWind=1):
        self.write(':CALC%d:MARK%d:MAX:NEXT'%(iWind,iNum))

    def Set_Mkr_On(self,iNum,iWind=1):
        self.write(':CALC%d:MARK%d ON'%(iWind,iNum))

    def Set_Mkr_Peak(self,iNum=1,iWind=1):
        self.write(':CALC%d:MARK%d:MAX:PEAK'%(iWind,iNum))

    def Set_Mkr_Time(self,fSec,iNum=1):
        self.write(':CALC1:MARK%d:X %fS'%(iNum,fSec))

    def Set_NoiseCorr(self,state):
        """0 1 ON OFF"""
        self.write(f':SENS:POW:NCOR {state}')

    def Set_Param_Couple_All(self):
        self.write("INST:COUP:CENT ALL")
        self.write("INST:COUP:RLEV ALL")
        self.write("INST:COUP:ATTEN ALL")
        self.write("INST:COUP:GAIN ALL")
        # self.query(":INST:COUP:USER1:NEW? 'Level','All Windows','Reference Level','LTE','All Windows','Reference Level',BID,ON")
        # self.query(":INST:COUP:USER2:NEW? 'Level','All Windows','Attenuation','LTE','All Windows','Attenuation',BID,ON")
        # self.query(":INST:COUP:USER3:NEW? 'Level','All Windows','Center Frequency','LTE','All Windows','Center Frequency',BID,ON")
        # self.query(":INST:COUP:USER4:NEW? 'Level','All Windows','Preamplifier','LTE','All Windows','Preamplifier',BID,ON")

        # self.query(":INST:COUP:USER1:NEW? 'Level','All Windows','Reference Level','IQACP','All Windows','Reference Level',BID,ON")
        # self.query(":INST:COUP:USER2:NEW? 'Level','All Windows','Attenuation','IQACP','All Windows','Attenuation',BID,ON")
        # self.query(":INST:COUP:USER3:NEW? 'Level','All Windows','Center Frequency','IQACP','All Windows','Center Frequency',BID,ON")
        # self.query(":INST:COUP:USER4:NEW? 'Level','All Windows','Preamplifier','IQACP','All Windows','Preamplifier',BID,ON")

    def Set_Preamp(self,sState):
        """0 1 ON OFF"""
        self.write('INP:GAIN:STAT %s;*WAI'%sState)

    def Set_PreampToggle(self,ChPwr,fToggle):
        if ChPwr < fToggle:     #FSVA:-23  FSW:-27
            self.Set_Preamp('ON')
        else:
            self.Set_Preamp('OFF')

    def Set_Ref_Source(self,sSource):
        """INT EXT E10 E100 E1000 EAUT"""
        self.write(f'ROSC:SOUR {sSource}')

    def Set_RefLevel(self,fReflevel):
        self.write('DISP:WIND:TRAC:Y:SCAL:RLEV %fdBm'%fReflevel)

    def Set_ResBW(self,fFreq):
        if fFreq == 0:
            self.write(':SENS:BAND:RES:AUTO ON')
        else:
            self.write(':SENS:BAND:RES %f'%fFreq)

    def Set_SamplingRate(self,fFreq):
        self.write('TRAC:IQ:SRAT %f'%fFreq)

    def Set_Savestate(self,sFilename):
        self.write(f':MMEM:STOR:STAT 1,"C:\\R_S\\Instr\\user\\{sFilename}"')

    def Set_Span(self,fFreq):
        self.write('SENS:FREQ:SPAN %f'%fFreq)

    def Set_SweepTime(self,fSwpTime):
        """Auto if fSwpTime == 0"""
        if fSwpTime == 0:
            self.write(':SENS:SWE:TIME:AUTO ON')
        else:
            self.write(f':SENS:SWE:TIME {fSwpTime}')             #Sweep/Capture Time

    def Set_SweepType(self,sType):
        #AUTO | SWE | FFT
        self.write(f':SENS:SWE:TYPE {sType}')

    def Set_SweepOpt(self,sOpt):
        #AUTO | SPEed | DYN
        self.write(f':SENS:SWE:OPT {sOpt}')

    def Set_SweepCont(self, sState):
        """0 | 1 """
        if sState in (1, '1', 'ON'):
            self.write('INIT:CONT ON')                      #Continuous Sweep
        elif sState in (0, '0', 'OFF'):
            self.write('INIT:CONT OFF')                     #Single Sweep

    def Set_SweepPoints(self,iNum):
        self.write(':SENS:SWE:POIN %f'%iNum)                #Number of trace points

    def Set_Trace_Avg(self,sType,trace=1):
        """LIN VID POW"""
        self.Set_Trace_Mode('AVER',trace)
        self.Set_Trace_Detector('AVER')
        self.Set_Trace_AvgType(sType)

    def Set_Trace_AvgCount(self,iAvg):
        self.write('SENS:SWE:COUN %d'%(iAvg))

    def Set_Trace_AvgOff(self,trace=1):
        self.write('DISP:TRAC%d:MODE WRIT'%(trace))

    def Set_Trace_AvgType(self,sType):
        """LIN VID POW"""
        self.write(f'SENS:AVER:TYPE {sType}')

    def Set_Trace_Detector(self,sDetect,iWind=1):
        """APE; NEG; POS; QPE; SAMP; RMS; AVER; CAV; CRMS"""
        self.write('SENS:WIND%d:DET %s'%(iWind,sDetect))

    def Set_Trace_Mode(self,sMode,trace=1):
        """WRIT AVER MAXH MINH VIEW BLAN"""
        self.write(f'DISP:TRAC{trace}:MODE {sMode}')

    ####################################################################
    ### FSW Trigger
    #####################################################################
    def Set_Trig1_Source(self,sDetect):
        """IMM; EXT; EXT2; EXT3; RFP; IFP; TIME; VID; PSEN"""
        self.write('TRIG:SEQ:SOUR %s'%sDetect)

    def Set_Trig2_Dir(self,sDir):
        """OUT | INP """
        if sDir == 'OUT':
            self.write('OUTP:TRIG2:DIR OUTP')
        else:
            self.write('OUTP:TRIG2:DIR INP')

    def Set_Trig2_OutType(self,sDir):
        """DEV : Device
        TARM: Trigger Armed
        UDEF: User Defined"""
        self.write(f'OUTP:TRIG2:OTYP {sDir}')

    def Set_VidBW(self,fFreq):
        if fFreq == 0:
            self.write(':SENS:BAND:VID:AUTO ON')
        else:
            self.write(':SENS:BAND:VID %f'%fFreq)

#####################################################################
### Run if Main
#####################################################################
if __name__ == "__main__":
    ### this won't be run when imported
    FSW = VSA().jav_Open("192.168.58.109")
    # print(FSW.Get_Param(1,0,1,1,1))
    asdf = FSW.Get_Mkr_Freq()
    FSW.jav_ClrErr()
    del FSW



##########################################################
### Rohde & Schwarz Automation for demonstration use.
###
### Purpose: Sweep FSW/SMW Frequncy
### Author:  mclim
### Date:    2018.05.17
##########################################################
### User Entry
##########################################################
import os
BaseDir = os.path.dirname(os.path.realpath(__file__))
OutFile = BaseDir + "\\data\\SMW_FSW_Sweep"

print(__file__)

SMW_IP = '192.168.1.115'                    #IP Address
FreqStart = int(51e9)
FreqStop = int(75e9)
FreqStep = int(1e9)
fSpan = 100e6
SWM_Out = -20
Mixer = 1

##########################################################
### Code Start
##########################################################
from rssd.VSG.Common import VSG
from rssd.FileIO     import FileIO

f = FileIO()
DataFile = f.Init(OutFile)
SMW = VSG()
SMW.jav_Open(SMW_IP,f.sFName)

##########################################################
### Instrument Settings
##########################################################
SMW.Set_RFPwr(SWM_Out)                    #Output Power
SMW.Set_RFState('ON')                     #Turn RF Output on

f.write(SMW.query('FREQ:MULT:EXT:TYPE?'))   #SMZ #
f.write(SMW.query('FREQ:MULT:EXT:SNUM?'))   #Serial Num
f.write(SMW.query('FREQ:MULT:EXT:LOAD:VERS?'))
f.write(SMW.query('FREQ:MULT:EXT:FMAX?'))
f.write(SMW.query('FREQ:MULT:EXT:FMIN?'))
f.write(SMW.query('FREQ:MULT:EXT:REV?'))    #Revision

f.write("Power")
f.write(SMW.query('FREQ:MULT:EXT:PMAX?'))    #Revision
f.write(SMW.query('FREQ:MULT:EXT:PMIN?'))    #Revision
f.write(SMW.query('FREQ:MULT:EXT:STAT?'))
SMW.write("MMEM:CDIR '/smz/firmware/'")
f.write(SMW.query("FREQ:MULT:EXT:FIRM:CAT?"))
f.write(SMW.query("FREQ:MULT:EXT:CORR:POW:POIN?"))


SMW.jav_ClrErr()                          #Clear Errors



"""rssd.self.OTA.common.py driver test"""
import unittest
from rssd.OTA.ATS1800 import OTA

host = '192.168.1.50'                           #Get local machine name

class TestGeneral(unittest.TestCase):
    def setUp(self):                            #run before each test
        self.ATS1800 = OTA().open(host, type='test')

    def tearDown(self):                         #Run after each test
        self.assertEqual(self.ATS1800.SCPI_error(self)[0],'0')
        self.ATS1800.close()

    def test_OTA_Init(self):
        self.ATS1800.Init_Measurement()

    def test_OTA_Azimuth(self):
        setVal = 10
        self.ATS1800.Set_AzimuthSpeed(setVal)
        getVal = self.ATS1800.Get_AzimuthSpeed()
        if self.ATS1800.connected : self.assertEqual(setVal, getVal)
        setVal = 10
        self.ATS1800.Set_AzimuthAngle(setVal)
        getVal = self.ATS1800.Get_AzimuthAngle()
        getVal = self.ATS1800.Get_AzimuthRunning()
        if self.ATS1800.connected : self.assertEqual(setVal, getVal)

    def test_OTA_Elevation(self):
        setVal = 10
        self.ATS1800.Set_ElevateSpeed(setVal)
        getVal = self.ATS1800.Get_ElevateSpeed()
        if self.ATS1800.connected : self.assertEqual(setVal, getVal)
        setVal = 10
        self.ATS1800.Set_ElevateAngle(setVal)
        getVal = self.ATS1800.Get_ElevateAngle()
        getVal = self.ATS1800.Get_ElevateRunning()
        if self.ATS1800.connected : self.assertEqual(setVal, getVal)

    def test_OTA_Stop(self):
        self.ATS1800.Set_AzimuthStop()
        self.ATS1800.Set_ElevateStop()

    def test_OTA_SystemStat(self):
        self.ATS1800.Get_SysStat()

if __name__ == '__main__':
#coverage run -a -m unittest -b -v test_HW_OTA_Common
    suite = unittest.TestLoader().loadTestsFromTestCase(TestGeneral)
    unittest.TextTestRunner(verbosity=4).run(suite)



"""Vector Network Analyzer Common Functions"""
import math
from rssd.yaVISA import jaVisa            # pylint: disable=E0611,E0401

class VNA(jaVisa):
    """ Rohde & Schwarz Vector Network Analyzer Object """
    def __init__(self):
        super(VNA,self).__init__()          #Python2/3
        self.Model = "VNA"
        self.dChan = 1

    def __complex2dB__(self, data):
        """Calculate Re/Im --> dB"""
        outArry = []
        if len(data) % 2 == 0:
            for i in range(len(data)//2):
                magnitude = math.sqrt(data[2*i] ** 2 + data[2*i+1] ** 2)
                outArry.append(20 * math.log10(magnitude))
        else:
            print('Need even elements')
        return outArry

    def __complex2phase__(self, data):
        """Calculate Re/Im --> phase"""
        outArry = []
        if len(data) % 2 == 0:
            for i in range(len(data)//2):
                phase = math.degrees(math.atan(data[2*i+1]/data[2*i])) 
                outArry.append(phase)
        else:
            print('Need even elements')
        return outArry

    def __complex2dBm__(self, data):
        """Calculate Re/Im --> dBm"""
        outArry = []
        if len(data) % 2 == 0:
            for i in range(len(data)//2):
                magnitude = math.sqrt((data[2*i] ** 2 + data[2*i+1] ** 2)/50)
                outArry.append(20 * math.log10(magnitude) + 30)
        else:
            print('Need even elements')
        return outArry

    #####################################################################
    ### VNA GET Functions Alphabetical
    #####################################################################
    def Get_Pwrcal_Rx_State(self):
        rdStr = self.query(f':SENS:CORR:STAT?')
        return rdStr

    def Get_Pwrcal_State(self):
        rdStr = self.query(f':SOUR:POW:CORR:STAT?')
        return rdStr

    def Get_Trace_Data(self):
        """Retrive Formatted Data: Data Only"""
        self.write('FORM ASCII')
        rdStr = self.queryFloatArry(f':CALC{self.dChan}:DATA:DALL? FDAT')
        return rdStr

    def Get_Trace_Data_All(self):
        """Retrive Formatted Data: Memory & Data"""
        self.write('FORM ASCII')
        rdStr = self.queryFloatArry(f':CALC{self.dChan}:DATA:ALL? FDAT')
        return rdStr

    # def Get_Trace_Data_CAll(self):
    #     """Retrive Formatted Data: nxn matrix S-Param"""
    #     self.write('FORM ASCII')
    #     rdStr = self.queryFloatArry(f':CALC{self.dChan}:DATA:CALL? FDAT')
    #     return rdStr

    def Get_Trace_Names(self):
        rdStr = self.query(f':CALC{self.dChan}:PAR:CAT?')
        return rdStr

    #####################################################################
    ### VNA SAVE Functions Alphabetical
    #####################################################################
    def Save_Cal(self, sFName):
        """ Save calibration to cal manager """
        if not sFName.lower().endswith(f'.cal'):
            sFName += ".cal"
        self.write(f':MMEM:STOR:CORR {self.dChan},"{sFName}"')

    def Save_Screen(self,sFName):
        """ Save Sceen to BMP """
        self.write('HCOP:DEV:LANG BMP')
        self.write(f'MMEM:NAME "{sFName}.BMP"')
        self.write('HCOP:DEST "MMEM"; :HCOP')

    def Save_State(self, sFName):
        """ Save State """
        self.write(f'MMEM:STOR:STAT 1,"{sFName}"')

    def Save_Trace_CSV(self, sFName):
        """Save Data in dB Phase CSV format"""
        # self.write(f'MMEM:STOR:TRAC:CHAN "{sTrace}","{sFName}.csv",FORM, LOGP, POIN, COMM')
        # self.write(f'MMEM:STOR:TRAC:CHAN "{sTrace}","{sFName}.csv",UNF, COMP, POIN, COMM')
        self.write(f"MMEM:STOR:TRAC:CHAN ALL,'{sFName}.csv', UNF, LOGP, POIN, COMM")

    def Save_Trace_SxP(self, sFName):                        #MMM
        """Save SxP data.  Ch must contain All (x^x) S-Parameter traces"""
        self.write(f"MMEM:STOR:TRAC:CHAN 1,'C:\\Rohde&Schwarz\\Nwa\\{sFName}.s2p'")
        #self.write(f':MMEM:STOR:TRAC:PORT %d,'%s.s2p',COMP,1,2"%(dChan,sFName))

    #####################################################################
    ### VNA SET Functions Alphabetical
    #####################################################################
    # def Set_Cal_Group(self,sName,dChan=1):                        #MMM
    #     #sName should end in '.cal'
    #     if not sName.lower().endswith(f'.cal'):
    #         sName += ".cal"
    #     self.write(f':MMEM:LOAD:CORR:RES %d,%s"%(dChan,sName))     #Resolve Cal Group
    #     self.write(f':MMEM:LOAD:CORR %s"%(sName))                  #Load cal group.

    def Set_FreqStart(self,fFreq):
        self.write(f':SENS{self.dChan}:FREQ:STAR {fFreq}')

    def Set_FreqStop(self,fFreq):
        self.write(f':SENS{self.dChan}:FREQ:STOP {fFreq}')            #RF Freq

    def Set_IFBW(self,fFreq):
        self.write(f'SENS{self.dChan}:BAND {fFreq}')

    def Set_InitImm(self):
        self.query(f'INIT:IMM;*OPC?')

    def Set_Mkr_Coupled(self, state):
        if state in (1,'1', 'ON'):
            self.write(f'CALC{self.dChan}:MARK:COUP ON')
        elif state in (0,'0', 'OFF'):
            self.write(f'CALC{self.dChan}:MARK:COUP OFF')

    def Set_Mkr_Frq(self, frq, mkr=1):
        # self.write(f'CALC{self.dChan}:MARK{mkr} ON')
        # self.write(f'CALC{self.dChan}:MARK{mkr} {frq}')
        self.write(f'CALC:MARK{mkr}:STAT ON')
        self.write(f'CALC:MARK{mkr}:X {frq:.0f}')

    def Set_PowerStart(self, fPwr):
        self.write(f':SOUR{self.dChan}:POW:STAR {fPwr} dBm')

    def Set_PowerStop(self, fPwr):
        self.write(f':SOUR{self.dChan}:POW:STOP {fPwr} dBm')

    def Set_Pwrcal_Init(self):
        self.write(f':SOUR:POW:CORR:COLL:FLAT ON')      #Flatness Cal
        self.write(f':SOUR:POW:CORR:COLL:RREC ON')      #Ref Rx Cal
        self.write(f':SOUR:POW:CORR:COLL:VER ON')       #Verification Sweep
        self.write(f':SOUR:POW:CORR:COLL:METH PMON')    #PMON | RFAF | RRON

    def Set_Pwrcal_NumReading(self, iNum):
        self.write(f':SOUR{self.dChan}:POW:CORR:NRE {iNum}')

    def Set_Pwrcal_Measure(self, iPort):
        self.write(f':SOUR{self.dChan}:POW:CORR:ACQ PORT,{iPort}')

    def Set_Pwrcal_Rx(self, Source, Port):
        # CORR:POW:ACQ <What to Cal> <Port>,<SourceTYpe>,<Port#>,<AWAV/NOM>
        self.write(f':CORR:POW:ACQ BWAV,{Port},PORT,{Source},AWAV')
        self.query('CORR:POW:AWAV?')

    def Set_Pwrcal_Tolerance(self,fTol):
        self.write(f':SOUR:POW:CORR:COLL:AVER:NTOL {fTol}')

    def Set_SweepCont(self,sState):
        if sState in (1, 'ON'):
            self.write(f'INIT:CONT ON')                              #Continuous Sweep
        elif sState in (0, 'OFF'):
            self.write(f'INIT:CONT OFF')                             #Single Sweep

    def Set_SweepPoints(self,dPoints):
        self.write(f':SENS{self.dChan}:SWE:POIN {dPoints}')              #RF Freq

    def Set_SweepTime(self,fSwpTime):
        """Seconds. 0=Auto"""
        if fSwpTime == 0:
            self.write(f':SENS{self.dChan}:SWE:TIME:AUTO ON')           #Auto
        else:
            self.write(f':SENS{self.dChan}:SWE:TIME {fSwpTime}')         #Sweep/Capture Time

    def Set_Trace_Avg(self,sState):
        if sState in (1, 'ON'):
            self.write(f':SENS{self.dChan}:AVER:STAT ON')
        if sState in (1, 'ON'):
            self.write(f':SENS{self.dChan}:AVER:STAT OFF')

    def Set_Trace_AvgCount(self,iAvg):
        self.write(f':SENS{self.dChan}:AVER:COUN {iAvg}')

    def Set_Trace_DelAll(self):
        self.write(f'CALC:PAR:DEL:ALL')

    def Set_Trace_MeasAdd(self,sMeas):
        # S11/S21/S12/S22 ..... Sxxyy
        # Y11/Y21/Y12/Y22 ..... Yxxyy
        # A1D2/A1D4/A2D1  ..... A<port>G<port>
        # B1D2/B1D4/B2D1  ..... B<port>G<port>
        # IP3UI/IP3UO      ..... IP<order:3|5|7|9><side:U|L><DUT:I|O>
        self.write(f'CALC{self.dChan}:PAR:SDEF "{sMeas}","{sMeas}"')     #<TrcName>,<Measurement>
        self.write(f'DISP:WIND1:TRAC:EFE "{sMeas}"')                #Displays Trace

    def Set_Trace_MeasAdd_AWave(self,APort,GenPort):
        #Default: SAM; RMS; PEAK; AVG
        self.Set_Trace_MeasAdd(f'A{APort}D{GenPort}RMS')

    def Set_Trace_MeasAdd_BWave(self,BPort,GenPort):
        self.Set_Trace_MeasAdd(f'B{BPort}D{GenPort}RMS')

    # def Set_Trace_MeasAdd_IMD3(self,dChan=1):                         #mmm
    #     self.write(f':SENS{dChan}:FREQ:IMOD:ORD3 ON"%(dChan))
    #     self.Set_Trace_MeasAdd(f'IP3UI')
    #     self.Set_Trace_MeasAdd(f'IP3LI')

    def Set_Trace_MeasAdd_SParam(self,Port1,Port2):
        """Port1,Port2"""
        self.Set_Trace_MeasAdd(f'S{Port1}{Port2}')

    def Set_Trace_MeasAdd_PwrMtr(self,GenPort):
        self.Set_Trace_MeasAdd(f'Pmtr{1}D{GenPort}')

    def Set_Trace_MeasDel(self,sTrcName):
        self.write(f'CALC{self.dChan}:PAR:DEL "{sTrcName}"')

    def Set_Trace_Select(self,sTrcName):
        self.write(f'CALC{self.dChan}:PAR:SEL "{sTrcName}"')

#####################################################################
### Run if Main
#####################################################################
if __name__ == "__main__":
    # this won't be run when imported
    ZVA = VNA().jav_openvisa('TCPIP0::192.168.58.30::inst0::INSTR')
    #ZVA.Test_PwrCal()
    data = ZVA.Get_Trace_Data()
    print(len(data))
    ZVA.jav_Close()
